
package fr.inria.infovis.panel;


import infovis.Graph;
import infovis.Table;
import infovis.Tree;
import infovis.Visualization;
import infovis.column.StringColumn;
import infovis.graph.DefaultGraph;
import infovis.graph.io.GraphReaderFactory;
import infovis.graph.io.HTMLGraphReader;
import infovis.graph.visualization.GraphVisualization;
import infovis.graph.visualization.MatrixVisualization;
import infovis.table.DefaultTable;
import infovis.table.io.TableReaderFactory;
import infovis.table.visualization.ScatterPlotVisualization;
import infovis.tree.DefaultTree;
import infovis.tree.io.TreeReaderFactory;
import infovis.tree.visualization.TreeVisualization;
import infovis.tree.visualization.TreemapVisualization;
import infovis.tree.visualization.treemap.SliceAndDice;
import infovis.visualization.VisualizationFactory;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import agile2d.AgileJFrame;

/**
 * @author raf
 *
 * A replacement of the MainFrameDecorator of Infovis
 * 
 * 
 */
public class MainFrameDecorator implements ActionListener {
    JFrame frame;
    JMenuBar menuBar;
    JMenu fileMenu;
    JMenu viewMenu;
    JMenu helpMenu;
    JFileChooser fileChooser;
    JTextField urlChooser;
    JDialog urlDialog;

    JMenu visualizationMenu;
    Table table;
    Visualization visualization = null;
    AbstractControlPanel controlPanel;
    JComponent splashScreen;
    boolean creatingWindow = true;
    boolean usingAgile = false;

    AbstractAction fileOpenAction = new DefaultAction("Open", 'O') {
        public void actionPerformed(ActionEvent e) {
            chooseFile();
        }
    };
    AbstractAction urlOpenAction = new DefaultAction("Url Open", 'U') {
        public void actionPerformed(ActionEvent e) {
            chooseURL();
        }
    };
    AbstractAction quitAction = new DefaultAction("Quit", 'Q') {
        public void actionPerformed(ActionEvent e) {
            System.exit(0);
        }
    };
    AbstractAction aboutAction = new AbstractAction("About...") {
        public void actionPerformed(ActionEvent e) {
            JOptionPane.showMessageDialog(
                null,
                "The Visualization Toolkit\n(C) Jean-Daniel Fekete and INRIA, France",
                "About the Visualization Toolkit",
                JOptionPane.INFORMATION_MESSAGE);
        }
    };

    AbstractAction toggleAgileAction =
        new AbstractAction("Toggle Agile2D") {
        public void actionPerformed(ActionEvent e) {
            usingAgile = !usingAgile;
        }
    };

    /**
     * Creates a new MainFrameDecorator object.
     *
     * @param frame the JFrame to decorate.
     */
    public MainFrameDecorator(JFrame frame) {
        this.frame = frame;
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        urlDialog = new JDialog(frame, "Open URL ...", true);
        urlChooser = new JTextField(30);
        urlChooser.setAutoscrolls(true);
        urlChooser.addActionListener(this);
        urlDialog.getContentPane().add(urlChooser);
        urlDialog.pack();
        fileChooser = new JFileChooser();
        fileChooser.setFileSelectionMode(
            JFileChooser.FILES_AND_DIRECTORIES);
        fileChooser.setCurrentDirectory(new File("."));
        fileChooser.setDoubleBuffered(false);
        menuBar = new JMenuBar();
        frame.setJMenuBar(menuBar);
        createMenus();
        frame.getContentPane().add(createSplashScreen());
    }

    /**
     * Creates the menus.
     */
    protected void createMenus() {
        fileMenu = createFileMenu();
        menuBar.add(fileMenu);
        viewMenu = createViewMenu();
        menuBar.add(viewMenu);
        visualizationMenu = new JMenu("Visualization");
        visualizationMenu.setEnabled(false);
        menuBar.add(visualizationMenu);
        helpMenu = createHelpMenu();
        menuBar.add(helpMenu);
    }

    /**
     * Creates the file menu.
     *
     * @return the file menu.
     */
    public JMenu createFileMenu() {
        JMenu filemenu = new JMenu("File");

        filemenu.add(fileOpenAction);
        filemenu.add(urlOpenAction);
        filemenu.addSeparator();
        filemenu.add(quitAction);
        return filemenu;
    }

    public JMenu createViewMenu() {
        JMenu viewmenu = new JMenu("View");

        JCheckBoxMenuItem cb = new JCheckBoxMenuItem(toggleAgileAction);
        cb.setArmed(usingAgile);
        viewmenu.add(cb);
        return viewmenu;
    }

    /**
     * Creates the help menu.
     *
     * @return the help menu.
     */
    public JMenu createHelpMenu() {
        JMenu helpmenu = new JMenu("Help");

        helpmenu.add(aboutAction);
        return helpmenu;
    }

    /**
     * Create the splash screen.
     *
     * @return the splash screen.
     */
    public JComponent createSplashScreen() {
        splashScreen = new SplashPanel();
        return splashScreen;
    }

    /**
     * shows a file chooser and wait for an selection.
     */
    protected void chooseFile() {
        int ret = fileChooser.showOpenDialog(null);
        if (ret == JFileChooser.APPROVE_OPTION) {
            File file = fileChooser.getSelectedFile();
            String error = openFile(file);
            if (error == null) {
                frame.pack();
            }
            else {
                JOptionPane.showMessageDialog(
                    null,
                    "Error",
                    "Couldn't read file " + file.getAbsoluteFile(),
                    JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    protected void chooseURL() {
        urlDialog.setVisible(true);
        String url = urlChooser.getText();
        if (url.length() != 0) {
            Graph graph = new DefaultGraph();
            HTMLGraphReader reader = new HTMLGraphReader(url, graph);
            reader.add(url);
            if (reader.load()) {
                visualization = createGraphVisualization(graph);
            }
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param file DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String openFile(File file) {
        String name = file.getAbsolutePath();

        table = new DefaultTable();
        Graph graph = new DefaultGraph(table);
        boolean loaded = GraphReaderFactory.readGraph(name, graph);

        if (loaded) {
            visualization = createGraphVisualization(graph);
            return null;
        }
        Tree tree = new DefaultTree(table);
        loaded = TreeReaderFactory.readTree(name, tree);
        if (loaded) {
            visualization = createTreeVisualization(tree);
            return null;
        }

        loaded = TableReaderFactory.readTable(name, table);
        if (loaded) {
            visualization = createTableVisualization(table);
            return null;
        }

        return "invalid format";
    }

    /**
     * DOCUMENT ME!
     *
     * @param t DOCUMENT ME!
     * @param index DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public static StringColumn getStringColumn(Table t, int index) {
        StringColumn ret = null;
        for (int i = 0; i < t.getColumnCount(); i++) {
            ret = StringColumn.getColumn(t, i);
            if (ret != null && !ret.isInternal() && index-- == 0)
                return ret;
        }
        return null;
    }

    /**
     * DOCUMENT ME!
     *
     * @param t DOCUMENT ME!
     * @param vis DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Visualization createContols(
        Table t,
        final Visualization vis) {
        //visualization.setLabelColumn(getStringColumn(table, 0));
        ControlPanelFactory factory =
            ControlPanelFactory.sharedInstance();

        final AbstractControlPanel control =
            (AbstractControlPanel) factory.createControlPanel(vis);
        if (splashScreen != null) {
            frame.getContentPane().remove(splashScreen);
        }
        if (controlPanel != null && !creatingWindow) {
            frame.getContentPane().remove(controlPanel);
            controlPanel.getVisualization().dispose();
            controlPanel = null;
        }
        else if (controlPanel != null && creatingWindow) {
            JFrame window =
                usingAgile
                    ? new AgileJFrame("Agile2D " + t.getName())
                    : new JFrame(t.getName());
            window.addWindowListener(new WindowAdapter() {
                public void windowClosed(WindowEvent e) {
                    control.dispose();
                }
            });
            window.setSize(500, 500);
            window.getContentPane().add(control);
            window.pack();
            window.setVisible(true);
            return vis;
        }
        controlPanel = control;
        frame.getContentPane().add(controlPanel);
        frame.pack();
        System.gc();
        System.runFinalization();
        VisualizationFactory.Creator[] creators =
            VisualizationFactory
                .sharedInstance()
                .getCompatibleCreators(
                t);
        visualizationMenu.removeAll();
        for (int i = 0; i < creators.length; i++) {
            addVisualizationMenu(creators[i], i);
        }
        visualizationMenu.setEnabled(creators.length != 0);

        return vis;
    }

    public void addVisualizationMenu(
        final VisualizationFactory.Creator creator,
        int i) {
        final String name = creator.getName();
        visualizationMenu.add(new DefaultAction(name, '1' + i) {
            public void actionPerformed(ActionEvent e) {
                Visualization vis = creator.create(table);
                if (vis != null)
                    createContols(table, vis);
            }
        });
    }

    /**
     * DOCUMENT ME!
     *
     * @param graph DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public GraphVisualization createGraphVisualization(Graph graph) {
        table = graph;
        MatrixVisualization vis =
            new MatrixVisualization(graph);
        createContols(graph, vis);
        return vis;
    }

    /**
     * DOCUMENT ME!
     *
     * @param tree DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public TreeVisualization createTreeVisualization(Tree tree) {
        table = tree;
        TreemapVisualization vis =
            new TreemapVisualization(
                tree,
                null,
                SliceAndDice.SLICEANDDICE);
        createContols(tree, vis);
        return vis;
    }

    /**
     * DOCUMENT ME!
     *
     * @param t DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Visualization createTableVisualization(Table t) {
        this.table = t;
        Visualization vis =
            new ScatterPlotVisualization(t);
        createContols(t, vis);
        return vis;
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == urlChooser) {
            urlDialog.setVisible(false);
        }

    }

    /**
     * Main program.
     *
     * @param args args.
     */
    public static void main(String[] args) {
        JFrame frame = new JFrame("InfoVis Toolkit");
        new MainFrameDecorator(frame);
        frame.pack();
        frame.setVisible(true);
    }
}
