/*
 * Created on 6 mai 2004
 *
 */
package fr.inria.infovis.temporizer;

import infovis.Column;
import infovis.Table;
import infovis.Tree;


/**
 * 
 * Specific Temporizer dedicated to Infovis Tree
 * 
 * @author raf
 *
 */
public class TreeTemporizer extends AbstractTemporizer {

	protected Tree tree;

	/**
	 * 
	 * Constructs a new temporizer for a tree
	 * 
	 * @param g graph to be "temporized"
	 */
	public TreeTemporizer(Tree t) {
		this(DEFAULT_DELAY, t);
	}
	/**
	 * 
	 * Constructs a new temporizer for a tree with a specific delay
	 * 
	 * @param d The delay in milliseconds
	 * @param g the graph to be temporized
	 */
	public TreeTemporizer(int d, Tree t) {
		super(d,t);
		this.tree = t;
	}
		
	
	
	
	/**
	 * 
	 * Disable notification on a Tree
	 * Basically disable notification on on Tree internal table.
	 * 
	 * @see fr.inria.infovis.temporizer.AbstractTemporizer#disableNotify()
	 */
	protected void disableNotify() {		
		Table t = tree.getTable();		
		Column c;
		for(int i=0; i < t.getColumnCount(); i++) {
			c=t.getColumnAt(i);
			c.disableNotify();
		}
		
		
	}
	/**
	 * 
	 * Enable notification on a Tree
	 * Basically enable notification on Tree internal table.
	 * 
	 * @see fr.inria.infovis.temporizer.AbstractTemporizer#disableNotify()
	 */
	protected void enableNotify() {
		Table t = tree.getTable();		
		Column c;
		for(int i=0; i < t.getColumnCount(); i++) {
			c=t.getColumnAt(i);
			try {
				c.enableNotify();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		
	}
	
}
