/*
 * Created on 5 avr. 2004
 *
 */
package fr.inria.oadymppac.application;





import javax.swing.JFrame;

import org.apache.log4j.Logger;

import fr.inria.oadymppac.io.FileManager;
import fr.inria.oadymppac.io.GenericTraceFileManager;
import fr.inria.oadymppac.panel.ApplicationPanelFactory;
import fr.inria.oadymppac.panel.MainActionFactory;
import fr.inria.oadymppac.panel.MainFrameDecorator;
import fr.inria.oadymppac.panel.PanelFactory;
import fr.inria.oadymppac.panel.VisualizationFrameDecorator;
import fr.inria.oadymppac.utils.DefaultProperties;
import fr.inria.oadymppac.utils.Properties;
import fr.inria.oadymppac.utils.log.LogConfigurator;



/**
 * 
 * The main application class
 * 
 * 
 * @author raf
 *
 */
public class MainApplication implements Properties, ApplicationProperties {
	/**
	 * <code>MAJOR_VERSION</code> the major version number of the application 
	 */
	private static final int MAJOR_VERSION=1;
	/**
	 *<code>MINOR_VERSION</code> the minor version number of the application
	 */
	private static final int MINOR_VERSION=0;
	/**
	 * <code>BUILD_VERSION</code> the build version of the application
	 */
	private static final int BUILD_VERSION=0;
	/**
	 * <code>INTERNAL_VERSION</code> the internal (CVS) version number
	 */
	private static final String INTERNAL_VERSION="$Id";
	/**
	 * <code>logger</code> the default logger of the application
	 */
	private static final Logger logger = Logger.getLogger(MainApplication.class);
	/**
	 *  <code>properties</code> the main property table of the application
	 */
	Properties properties;
	
    /**
     * <code>frame</code> the main JFrame of the application
     */
    static JFrame frame;
    /**
     * <code>mainDecorator</code> the main frame decorator of the application
     */
    MainFrameDecorator mainDecorator;
    /**
     * <code>frameDecorator</code> the visualization frame decorator of the application
     */
    VisualizationFrameDecorator frameDecorator;
    /**
     * <code>actionFactory</code> the action factory of the applciation
     */
    MainActionFactory actionFactory;
    /**
     * <code>panelFactory</code> the panel factory of the application
     */
    PanelFactory panelFactory;
    /**
     * <code>fileManager</code> the file I/O manager of the application
     */
    FileManager fileManager;
    
    
    /**
     * Default Constructor
     */
    public MainApplication() {
    	properties = new DefaultProperties();
    	LogConfigurator.configure();
    	setProperty(LOGGER,logger);
    	setProperty(APPLICATION,this);
    	
    	
    }
    
    /**
     * init all the sub part of the application
     */
    public void initApplication() {
    	mainDecorator = new MainFrameDecorator(properties,"OADYMPPAC Trace Analyzer");
    	frame = mainDecorator.getFrame();
    	setProperty(APPLICATION_FRAME_DECORATOR, mainDecorator);
    	mainDecorator = null; //j'en ai plus besoin !
    	setProperty(MAIN_FRAME, frame);
        actionFactory = new MainActionFactory(properties);
        setProperty(ACTION_FACTORY,actionFactory);
        fileManager = new GenericTraceFileManager(properties);
        setProperty(FILE_MANAGER,fileManager);
        panelFactory = new ApplicationPanelFactory(properties);
        setProperty(PANEL_FACTORY,panelFactory);
        frameDecorator= new VisualizationFrameDecorator(properties);
        setProperty(VISUALIZATION_FRAME_DECORATOR,frameDecorator);
    }
    
 
    /**
     * @return Returns the frame.
     */
    public static JFrame getFrame() {
        return frame;
    }
    /**
     * @return Returns the fileManager.
     */
    public FileManager getFileManager() {
        return (FileManager) getProperty(FILE_MANAGER);
    }
    /**
     * @return Returns the panelFactory.
     */
    public PanelFactory getPanelFactory() {
        return (PanelFactory) getProperty(PANEL_FACTORY);
    }
    /**
     * @param fileManager The fileManager to set.
     */
    public void setFileManager(FileManager fileManager) {
        this.fileManager = fileManager;
        setProperty(FILE_MANAGER, fileManager);
    }
	
	/**
	 * @return the property tezable of the application
	 */
	public Properties getProperties() {
		return properties.getProperties();
	}
	/**
	 * @param property the property name
	 * @return the property value if defined or null
	 */
	public Object getProperty(String property) {
		return properties.getProperty(property);
	}
	/**
	 * 
	 * set the property table of the application
	 * 
	 * @param properties to set
	 */
	public void setProperties(Properties properties) {
		properties.setProperties(properties);
	}
	/**
	 * 
	 * Set a given property
	 * 
	 * @param property the propety name
	 * @param value the value for the property
	 */
	public void setProperty(String property, Object value) {
		properties.setProperty(property, value);
	}
	/**
	 * @return Returns the BUILD_VERSION.
	 */
	public synchronized int getBuildVersion() {
		return BUILD_VERSION;
	}
	/**
	 * @return Returns the INTERNAL_VERSION.
	 */
	public synchronized String getInternalVersion() {
		return INTERNAL_VERSION;
	}
	/**
	 * @return Returns the MAJOR_VERSION.
	 */
	public synchronized int getMajorVersion() {
		return MAJOR_VERSION;
	}
	/**
	 * @return Returns the MINOR_VERSION.
	 */
	public synchronized int getMinorVersion() {
		return MINOR_VERSION;
	}
}
