/*
* INRIA-OADYMPPAC
* 
* Ownership: Copyright (c) 1990-2004 COSYTEC S.A.
* Author	: 	raf
*           	COSYTEC
*		    	4 rue Jean Rostand
* 				91893 Orsay Cedex, France
*				Tel. +33 1 60 19 37 38
*				Fax. +33 1 60 19 36 20
*
* Purpose	: COMPLETE HERE
* 
* Created on 10 mars 2004
*
* 
*/
package fr.inria.oadymppac.examples.parsingfaster;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.ProgressMonitorInputStream;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.log4j.Logger;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

import fr.inria.infovis.panel.VisualizationPanel;
import fr.inria.oadymppac.io.GenericTraceFileFilter;
import fr.inria.oadymppac.io.PostMortemInputSource;
import fr.inria.oadymppac.parsing.gentra4cp.handlers.FasterTraceContentHandler;
import fr.inria.oadymppac.utils.log.LogConfigurator;
import fr.inria.oadymppac.widget.SwingWorker;


/**
* @author raf
*
* @
*/
public class SingleParserExample {
    static Logger logger = Logger.getLogger(SingleParserExample.class);
    protected final static File DEFAULT_FOLDER=new File("data");

    static JFrame frame;
    
    public SingleParserExample() {
        logger.info("Building new instance of the application");
        //init_handlers();
    };
    
    public static void main(String[] args) {
        LogConfigurator.configure();
        frame = new JFrame("Parsing test");
        frame.setSize(200,200);
        frame.getContentPane().add(new VisualizationPanel(null));
        frame.setVisible(true);
        logger.info("Starting example program");
        SingleParserExample example = new SingleParserExample();
        logger.info("Number of arguments:"+args.length);
        if (args.length < 1) {
            example.selectFile();
        } 
        System.exit(0);
    }
    
    
    protected void selectFile() {
    	JFileChooser fileChooser = new JFileChooser();
    	fileChooser
        .setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);

    	fileChooser.setCurrentDirectory((DEFAULT_FOLDER.exists() ? DEFAULT_FOLDER : new File(".")));
    	fileChooser.setFileFilter(new GenericTraceFileFilter());
    	fileChooser.setDoubleBuffered(false);

    	int ret = fileChooser.showOpenDialog(null);
    	if (ret == JFileChooser.APPROVE_OPTION) {
    		 //parse(fileChooser.getSelectedFile().getAbsolutePath());
    		ParserThread loader = new ParserThread(fileChooser.getSelectedFile().getAbsolutePath());
    		//loader.run();
    		loader.construct();
    	} 
    }
    
    class ParserThread /*implements Runnable*/ extends SwingWorker {
    	String filename;
        SAXParser parser;
        XMLReader reader;
        DefaultHandler defaultHandler;
        FasterTraceContentHandler mainHandler;  
        
        SingleHeadEventListener headListener;
        SingleProvideEventListener providableListener, providedListener;
        SingleControlEventListener controlListener;
        SingleComplementEventListener complementListener;
        SingleSolverEventListener solverListener;

    	
    	public ParserThread(String file) {
    		filename=file;
    		init_handlers();
    	}
    	
    	
    	/*public void run() {
    		parse(filename);
    	}
    	*/
    	public Object  construct() {
    		parse(filename);
    		return null;
    	}
    	
    	
    protected void parse(String fname){
 
    	long time = System.currentTimeMillis();
        if (logger.isDebugEnabled()) logger.info("Parsing XML file:"+fname);
        try {
            InputSource is=new PostMortemInputSource(fname);
            InputStream pgs = is.getByteStream();
            InputStream progress = new ProgressMonitorInputStream(
                    null,
                    "Reading " + fname,
                    pgs);
        	is.setByteStream(progress);
            parser = SAXParserFactory.newInstance().newSAXParser();
            defaultHandler=new DefaultHandler();
            logger.info("New SAXParser created");
            reader = parser.getXMLReader();
            reader.setContentHandler(mainHandler);
            reader.setDTDHandler(defaultHandler);
            reader.setEntityResolver(defaultHandler);
            reader.setErrorHandler(defaultHandler);
            reader.parse(is);
            System.out.println("End of Parsing, garbaging");
            System.gc();
        } catch (SAXException e) {
            logger.error("SAX Exception",e);
        } catch (FileNotFoundException e) {
            if (logger.isDebugEnabled()) {
                logger.error("FILE: " + fname + " cannot be found", e);
            }
        } catch (IOException e) {
            logger.error("I/O Exception",e);
        } catch (ParserConfigurationException e) {
            logger.error("Parser Configuration Exception",e);
        } finally {
            logger.info("Finally clause");
        }
        System.out.println("PROVIDED:"+ mainHandler.getHeadProvideInfo());
        solverListener.show();
        JOptionPane.showMessageDialog(null,"Parsing is ended successfully in "+(System.currentTimeMillis()-time)+" ms.");
    }
    
    protected void init_handlers() {
        logger.info("Initializing default handler");
        mainHandler = new FasterTraceContentHandler();
        
        headListener = new SingleHeadEventListener();
        providableListener = new SingleProvideEventListener(SingleProvideEventListener.HEAD);
        providedListener = new SingleProvideEventListener(SingleProvideEventListener.TRACE);
        controlListener = new SingleControlEventListener();
        complementListener = new SingleComplementEventListener();
        solverListener = new SingleSolverEventListener();
        
        
        
        mainHandler.addDefaultListener(solverListener);
        
        mainHandler.addControlListener(controlListener);
        
        mainHandler.addHeadListener(headListener);
        
        mainHandler.addProvidableListener(providableListener);
        mainHandler.addProvidedListener(providedListener);
        mainHandler.addComplementListener(complementListener);
        
        
    } 
        
 
    }
    
    
    
}
