/*
 * Created on 30 oct. 2003
 */
package fr.inria.oadymppac.io;

import java.io.File;

import fr.inria.oadymppac.parsing.ContentHandler;
import fr.inria.oadymppac.parsing.event.EventListener;
import fr.inria.oadymppac.parsing.gentra4cp.handlers.FasterTraceContentHandler;
import fr.inria.oadymppac.utils.Properties;


/**
 * 
 * 
 * This class extends the Standard file loader thread using a 
 * less generinc but more efficient ContentHandler.
 * 
 * The parsing gains efficiency by dividing by 5 (in some cases)
 * the time to parse a file.
 * 
 * 
 * @author raf
 *
 */
public class FasterFileLoaderThread extends FileLoaderThread {

	
	
	
	/** 
	 * 
	 * Build a FasterTraceContentHandler instead of a standard TraceContentHandler
	 * 
	 * @see fr.inria.oadymppac.io.FileLoaderThread#buildMainHandler()
	 */
	protected ContentHandler buildMainHandler() {
    	return new FasterTraceContentHandler();
    }
    
    /**
     * 
     * Configure the Handler and the event listeners according to the 
     * FasterTraceContentHandler 
     * 
     * @see fr.inria.oadymppac.io.FileLoaderThread#initMainHandler()
     */
    protected void initMainHandler() {	
    	FasterTraceContentHandler handler = (FasterTraceContentHandler) mainHandler;
    	handler.addControlListener(controlListener);
    	handler.addHeadListener(headListener);
    	handler.addProvidableListener(providableListener);
    	handler.addProvidedListener(providedListener);
    	handler.addComplementListener(complementListener);
    }
    
    
    
    
    /**
     * 
     * Add an additional listener to process TraceEvent
     * 
     * @see fr.inria.oadymppac.io.FileLoaderThread#addListener(fr.inria.oadymppac.parsing.event.EventListener)
     */
    public void addListener(EventListener listener) {
    	
    	if (mainHandler == null) init_handlers();
        ((FasterTraceContentHandler)mainHandler).addDefaultListener(listener);
    }
    /**
    
}
	/**
	 * 
	 * Constructor 
	 *
	 * @param properties  property tabe to configure the thread
	 * @param file file to parse
	 */
	public FasterFileLoaderThread(Properties properties, File file) {
		super(properties, file);
	}
}
