/*
 * Created on 5 avr. 2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package fr.inria.oadymppac.panel;

import infovis.panel.DefaultAction;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.AbstractAction;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;

import fr.inria.infovis.panel.VisualizationPanel;
import fr.inria.oadymppac.application.ApplicationProperties;
import fr.inria.oadymppac.io.FileManager;
import fr.inria.oadymppac.solver.Solver;
import fr.inria.oadymppac.solver.analyser.ConstraintConstraintGraphAnalyser;
import fr.inria.oadymppac.solver.analyser.SolverAnalyser;
import fr.inria.oadymppac.solver.analyser.VariableConstraintGraphAnalyser;
import fr.inria.oadymppac.utils.Properties;

/**
 * @author raf
 * 
 * To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Generation - Code and Comments
 */
public class MainActionFactory {
	public static final long DEFAULT_ACTION_TIME=200; //Half second before starting action;
	protected Properties properties;
	
	protected AbstractAction aboutAction=null;
	protected AbstractAction openAction=null;
	protected AbstractAction quitAction=null;
	protected AbstractAction splashAction=null;
	protected AbstractAction analyseAction=null;
	protected AbstractAction tileHorizontallyAction=null;
	protected AbstractAction tileVerticallyAction=null;
	protected AbstractAction mosaicAction=null;
	protected AbstractAction cascadeAction=null;
	protected AbstractAction zoomInAction=null;
	protected AbstractAction zoomOutAction=null;
	
	protected TimerTask aboutTask=null;
	protected TimerTask openTask=null;
	protected TimerTask quitTask=null;
	protected TimerTask splashTask=null;
	protected TimerTask analyseTask=null;
	protected TimerTask tileHorizontallyTask=null;
	protected TimerTask tileVerticallyTask=null;
	protected TimerTask mosaicTask=null;
	protected TimerTask cascadeTask=null;
	protected TimerTask zoomInTask=null;
	protected TimerTask zoomOutTask=null;
	protected boolean splashMain=false;
	
	protected Timer actionTimer;
	
	public MainActionFactory(Properties properties) {
		this.properties = properties;
		//properties.setProperty(ApplicationProperties.APPLICATION, application);
		actionTimer = new Timer();
	}
	
	public AbstractAction createAboutAction() {
		if (aboutAction == null) {
			aboutAction= new AbstractAction("About...") {			
				public void actionPerformed(ActionEvent e) {
					aboutTask=new TimerTask() {
						public void run() {
							PanelFactory panelFactory = (PanelFactory) properties.getProperty(ApplicationProperties.PANEL_FACTORY);
							panelFactory.buildAboutPanel();
						}
					};
					actionTimer.schedule(aboutTask,DEFAULT_ACTION_TIME);
				}
			};
			aboutAction.putValue(ApplicationProperties.APPLICATION, properties.getProperty(ApplicationProperties.APPLICATION));
		}
		return aboutAction;
	}
	
	public AbstractAction createSplashAction() {
		if (splashAction == null) {
			splashAction= new AbstractAction("Info...") {
				public void actionPerformed(ActionEvent e) {
					splashTask = new TimerTask() {
						public void run() {
							PanelFactory panelFactory = (PanelFactory) properties.getProperty(ApplicationProperties.PANEL_FACTORY);
							JFrame frame = (JFrame) properties.getProperty(ApplicationProperties.MAIN_FRAME);
							if (splashMain) {
								JOptionPane.showConfirmDialog(frame,panelFactory.buildSplashScreen());
							} else {
								(frame.getContentPane()).add(panelFactory.buildSplashScreen(), BorderLayout.CENTER);
								splashMain=true;
							}
						}
					};
					//actionTimer.schedule(splashTask,DEFAULT_ACTION_TIME);
					splashTask.run(); // must be done immediately
				}
			};
			splashAction.putValue(ApplicationProperties.APPLICATION, properties.getProperty(ApplicationProperties.APPLICATION));
		}
		return splashAction;
	}
	
	
	
	public AbstractAction createOpenAction() {
		if (openAction == null ) {
			openAction= new DefaultAction("Open",'O') {
				public void actionPerformed(ActionEvent e) {
					openTask = new TimerTask() {
						public void run() {
							ApplicationPanelFactory panelFactory = (ApplicationPanelFactory) properties.getProperty(ApplicationProperties.PANEL_FACTORY);
							FileManager fileManager = (FileManager) properties.getProperty(ApplicationProperties.FILE_MANAGER);
							File file = fileManager.selectFile();
							if (file != null) {               	
								JListDialog.setContent(SolverAnalyser.LABELS);
								if (panelFactory.buildAnalysisSelector() == JOptionPane.OK_OPTION) {
									try {
										fileManager.loadFile(file);
									} catch (Exception excp) {
										panelFactory.buildErrorPanel(excp);
									}  
								}
							}                
						}
					};
					actionTimer.schedule(openTask, DEFAULT_ACTION_TIME);
				}
			};
			openAction.putValue(ApplicationProperties.APPLICATION, properties.getProperty(ApplicationProperties.APPLICATION));
		}
		return openAction;
	}
	
	
	
	
	
	
	
	public AbstractAction createQuitAction() {
		if (quitAction == null) {
			quitAction = new DefaultAction("Quit", 'Q') {
				public void actionPerformed(ActionEvent e) {
					quitTask = new TimerTask() {
						public void run() {
							ApplicationPanelFactory panelFactory = (ApplicationPanelFactory) properties.getProperty(ApplicationProperties.PANEL_FACTORY);
							if (panelFactory.buildQuestionPanel("Do you really want to quit?") == JOptionPane.YES_OPTION)
								System.exit(0);
						}
					};
					actionTimer.schedule(quitTask,DEFAULT_ACTION_TIME );
				}
			};
			quitAction.putValue(ApplicationProperties.APPLICATION, properties.getProperty(ApplicationProperties.APPLICATION));
		}
		return quitAction;
	}
	
	public AbstractAction getDefaultAction() {
		return createSplashAction();
	}
	
	public AbstractAction createAnalyseAction() {
		if (analyseAction == null) {
			analyseAction = new DefaultAction("New Analysis", 'A') {
				public void actionPerformed(ActionEvent e) {
					analyseTask = new TimerTask() {
						public void run() {
							ApplicationPanelFactory panelFactory = (ApplicationPanelFactory) properties.getProperty(ApplicationProperties.PANEL_FACTORY);
							Solver solver = (Solver) properties.getProperty(ApplicationProperties.CURRENT_DOCUMENT);
							System.out.println("Solver:"+solver);
							JListDialog.setContent(SolverAnalyser.LABELS);
							if (panelFactory.buildAnalysisSelector() == JOptionPane.OK_OPTION) {
								Object[] analysers = JListDialog.getSelection();
								for (int i=0; i < analysers.length; i++) {
									if (analysers[i] == ConstraintConstraintGraphAnalyser.LABEL) {
										ConstraintConstraintGraphAnalyser constraintAnalyser = new ConstraintConstraintGraphAnalyser(solver);
										constraintAnalyser.analyse();
										panelFactory.buildVisualization(constraintAnalyser.getGraph());
									}
									if (analysers[i] == VariableConstraintGraphAnalyser.LABEL) {
										VariableConstraintGraphAnalyser variableAnalyser = new VariableConstraintGraphAnalyser(solver);
										variableAnalyser.analyse();
										panelFactory.buildVisualization(variableAnalyser.getGraph());
									}
								}       
								System.gc(); // should make a cleaning stuff !
								
							}
							
						}
					};
					actionTimer.schedule(analyseTask,DEFAULT_ACTION_TIME);
				}
			};
			analyseAction.putValue(ApplicationProperties.APPLICATION, properties.getProperty(ApplicationProperties.APPLICATION));
		}
		return analyseAction;
	}
	
	
	
	public AbstractAction createTileHorizontallyAction() {
		if (tileHorizontallyAction == null) {
			tileHorizontallyAction = new DefaultAction("Tile Horizontally", 'H') {
				public void actionPerformed(ActionEvent e) {
					tileHorizontallyTask = new TimerTask() {
						public void run() {
							VisualizationFrameDecorator deco = (VisualizationFrameDecorator) properties.getProperty(ApplicationProperties.VISUALIZATION_FRAME_DECORATOR);
							JDesktopPane desktop = (deco==null ? null : deco.getDesktop());
							Dimension d = (desktop == null ? null : desktop.getSize());
							JInternalFrame[] windows= (desktop== null ? null :desktop.getAllFrames());
							if (d != null && windows != null && windows.length >0) {
								int height = d.height / windows.length;
								for(int i = 0; i < windows.length; i++) {
									windows[i].setBounds(0,i*height,d.width,height);
								}
							}
						}
					};
					actionTimer.schedule(tileHorizontallyTask,DEFAULT_ACTION_TIME );
				}
			};
			tileHorizontallyAction.putValue(ApplicationProperties.APPLICATION, properties.getProperty(ApplicationProperties.APPLICATION));
		}
		return tileHorizontallyAction;
	}
	
	
	
	public AbstractAction createTileVerticallyAction() {
		if (tileVerticallyAction == null) {
			tileVerticallyAction = new DefaultAction("Tile Vertically", 'V') {
				public void actionPerformed(ActionEvent e) {
					tileVerticallyTask = new TimerTask() {
						public void run() {
							VisualizationFrameDecorator deco = (VisualizationFrameDecorator) properties.getProperty(ApplicationProperties.VISUALIZATION_FRAME_DECORATOR);
							JDesktopPane desktop = (deco==null ? null : deco.getDesktop());
							Dimension d = (desktop == null ? null : desktop.getSize());
							JInternalFrame[] windows= (desktop== null ? null : desktop.getAllFrames());
							if (d != null && windows != null && windows.length >0) {
								int width = d.width / windows.length;
								for(int i = 0; i < windows.length; i++) {
									windows[i].setBounds(i*width,0,width,d.height);
								}
							} 			
						}
					};
					actionTimer.schedule(tileVerticallyTask,DEFAULT_ACTION_TIME );
				}
			};
			tileVerticallyAction.putValue(ApplicationProperties.APPLICATION, properties.getProperty(ApplicationProperties.APPLICATION));
		}
		return tileVerticallyAction;
	}
	
	public AbstractAction createMosaicAction() {
		if (mosaicAction == null) {
			mosaicAction = new DefaultAction("Mosaic", 'M') {
				public void actionPerformed(ActionEvent e) {
					mosaicTask = new TimerTask() {
						public void run() {
							VisualizationFrameDecorator deco = (VisualizationFrameDecorator) properties.getProperty(ApplicationProperties.VISUALIZATION_FRAME_DECORATOR);
							JDesktopPane desktop = (deco==null ? null : deco.getDesktop());
							Dimension d = (desktop == null ? null : desktop.getSize());
							JInternalFrame[] windows= (desktop== null ? null :desktop.getAllFrames());
							if (d != null && windows != null && windows.length >0) {
								int row = (int) Math.sqrt(windows.length);
								int column = windows.length / row;
								if (column * row < windows.length) row++;
								int width = d.width / column;
								int height = d.height / row;
								for(int i = 0; i < windows.length; i++) {
									windows[i].setBounds((i % column) *width,(i / column)* height,width,height);
								}
							}					
						}
					};
					actionTimer.schedule(mosaicTask, DEFAULT_ACTION_TIME);
				}
			};
			mosaicAction.putValue(ApplicationProperties.APPLICATION, properties.getProperty(ApplicationProperties.APPLICATION));
		}
		return mosaicAction;
	}
	public AbstractAction createCascadeAction() {
		if (cascadeAction == null) {
			cascadeAction = new DefaultAction("Cascade", 'C') {
				public void actionPerformed(ActionEvent e) {
					cascadeTask = new TimerTask() {
						public void run() {
							VisualizationFrameDecorator deco = (VisualizationFrameDecorator) properties.getProperty(ApplicationProperties.VISUALIZATION_FRAME_DECORATOR);
							JDesktopPane desktop = (deco==null ? null : deco.getDesktop());
							Dimension d = (desktop == null ? null : desktop.getSize());
							JInternalFrame[] windows= (desktop== null ? null :desktop.getAllFrames());
							if (d != null && windows != null && windows.length >0) {
								int gap = 20;
								int width = d.width - gap*windows.length;
								int height = d.height - gap*windows.length;
								for(int i = 0; i < windows.length; i++) {
									windows[windows.length-i-1].setBounds(i*gap,i*gap,width,height);
								}
							}				
						}
					};
					actionTimer.schedule(cascadeTask, DEFAULT_ACTION_TIME);
				}
			};
			cascadeAction.putValue(ApplicationProperties.APPLICATION, properties.getProperty(ApplicationProperties.APPLICATION));
		}
		return cascadeAction;
	}
	public AbstractAction createZoomInAction() {
		if (zoomInAction == null) {
			zoomInAction = new DefaultAction("Zoom in", '+') {
				public void actionPerformed(ActionEvent e) {
					zoomInTask = new TimerTask() {
						public void run() {
							VisualizationInternalFrame visu = (VisualizationInternalFrame) properties.getProperty(ApplicationProperties.CURRENT_VISUALIZATION);
							VisualizationPanel panel = (VisualizationPanel) (visu == null ? null : visu.getVisualizationPanel());
							Dimension d = (panel == null ? null : panel.getPreferredSize());
							if (panel != null && d!=null) {
								d.width = Math.min((int) (d.width * 2), Integer.MAX_VALUE);
								d.height = Math.min((int) (d.height * 2 ), Integer.MAX_VALUE);
								panel.setPreferredSize(d);
								visu.forceRepaint();
							}
						}
					};
					actionTimer.schedule(zoomInTask, DEFAULT_ACTION_TIME);
				}
			};
			zoomInAction.putValue(ApplicationProperties.APPLICATION, properties.getProperty(ApplicationProperties.APPLICATION));
		}
		return zoomInAction;
	}
	public AbstractAction createZoomOutAction() {
		if (zoomOutAction == null) {
			zoomOutAction = new DefaultAction("Zoom out", '-') {
				public void actionPerformed(ActionEvent e) {
					zoomOutTask = new TimerTask() {
						public void run() {
							VisualizationInternalFrame visu = (VisualizationInternalFrame) properties.getProperty(ApplicationProperties.CURRENT_VISUALIZATION);
							VisualizationPanel panel = (VisualizationPanel) (visu == null ? null : visu.getVisualizationPanel());
							Dimension d = (panel == null ? null : panel.getPreferredSize());
							if (panel != null && d!=null) {
								d.width = Math.max((int) (d.width / 2), 100);
								d.height = Math.max((int) (d.height / 2 ), 100);
								panel.setPreferredSize(d);
								visu.forceRepaint();
							}					
						}
					};
					actionTimer.schedule(zoomOutTask, DEFAULT_ACTION_TIME);
				}
			};
			zoomOutAction.putValue(ApplicationProperties.APPLICATION, properties.getProperty(ApplicationProperties.APPLICATION));
		}
		return zoomOutAction;
	}
	
	
	
	protected void notYetImplemented() {
		ApplicationPanelFactory panelFactory = (ApplicationPanelFactory) properties.getProperty(ApplicationProperties.PANEL_FACTORY);
		panelFactory.buildMessagePanel("Not yet implemented !") ;
	}
	
}
