/*
 * Created on 5 avr. 2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package fr.inria.oadymppac.panel;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.Timer;

import org.apache.log4j.Logger;

import fr.inria.oadymppac.application.ApplicationProperties;
import fr.inria.oadymppac.utils.Properties;


/**
 * @author raf
 *
 * To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class MainFrameDecorator  {
	
    public static Logger logger = Logger.getLogger(MainFrameDecorator.class);

    protected Properties properties;
    protected JFrame frame;
    protected String application_frame_title;
    protected Timer message_timer;
    protected JLabel footerLabel=null;
    
    public static final String FILE_MENU="File";
    public static final String WIND_MENU="Window";
    public static final String HELP_MENU="Help";
    public static final String READY="Ready";
   
    public static final int MESSAGE_DELAY = 5000; // After 5 seconds idle, the message is hidden
   
    /**
     * @throws java.awt.HeadlessException
     */
    public MainFrameDecorator(Properties properties) throws HeadlessException {
        super();
        this.frameInit(properties,""); 
    }
    
    /**
     * @param title
     * @throws java.awt.HeadlessException
     */
    public MainFrameDecorator(Properties properties,String title)  {
        this.frameInit(properties, title); 
    }
    
    
    protected void frameInit(Properties props, String title ) {
        this.properties=props;
        frame = (JFrame) props.getProperty(ApplicationProperties.MAIN_FRAME); 
        if (frame==null) {
        	String classname = (String) props.getProperty(ApplicationProperties.ACCELERATED_FRAME_CLASS);
        	if (classname != null) {
        		try {
        			Class c = Class.forName(classname);
        			frame = (JFrame) c.newInstance();
        			if (classname == "agile2d.AgileJFrame") {
        				title = "Agile 2D "+title;
        			} else {
        				title = classname + "title";
        			}
        		} catch (ClassNotFoundException e) {
        			e.printStackTrace();
        			frame = new JFrame(title);
        		} catch (InstantiationException e) {
        			e.printStackTrace();
        			frame = new JFrame(title);
        		} catch (IllegalAccessException e) {
        			e.printStackTrace();
        			frame = new JFrame(title);
        //		} finally {
        //			frame = new JFrame(title);
        		}
    		} else {
    			frame = new JFrame(title);
    		}
        } 
        frame.setTitle(title);
        application_frame_title = title;
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        createMenuBar();
        frame.getRootPane().getContentPane().setLayout(new BorderLayout());
        createFooterPanel();
    }
    
    
    
    protected void createMenuBar() {
       JMenuBar menuBar = new JMenuBar();
       logger.info("Adding default menubar"+frame.getRootPane());
       frame.setJMenuBar(menuBar);
       JMenu fileMenu = new JMenu(FILE_MENU);
       JMenu windMenu = new JMenu(WIND_MENU);
       JMenu helpMenu = new JMenu(HELP_MENU);
       menuBar.add(fileMenu);
       menuBar.add(windMenu);
       menuBar.add(helpMenu);
       
    }
    
    protected void createFooterPanel() {
        final JPanel footerPanel = new JPanel();
        footerPanel.setLayout(new BorderLayout());
        footerPanel.setBorder(BorderFactory.createLoweredBevelBorder());
//        final JProgressBar progressBar = new JProgressBar(JProgressBar.HORIZONTAL, 0, Integer.MAX_VALUE) {
//    	    public Dimension getPreferredSize() {
//    		return new Dimension(100, Math.max(20,super.getPreferredSize().height));
//    	    }
//    	};
//    	progressBar.setMinimumSize(new Dimension(20,20));
//    	progressBar.setBorder(BorderFactory.createRaisedBevelBorder());
//        footerPanel.add(progressBar, BorderLayout.EAST);
        footerLabel= new JLabel(READY){
            public Dimension getPreferredSize() {
        		return new Dimension(100, Math.max(20,super.getPreferredSize().height));
        	    }
        };
        
        
        footerPanel.add(footerLabel, BorderLayout.CENTER);
        footerPanel.setMinimumSize(new Dimension(300,20));
        frame.getContentPane().add(footerPanel, BorderLayout.SOUTH);
        
        
    }
    
    public synchronized void setMessage(String message) {
    	footerLabel.setText(message);
    	Rectangle oldBounds = footerLabel.getBounds();
    	Rectangle newBounds = new Rectangle();
    	newBounds.x = oldBounds.x;
    	newBounds.y = oldBounds.y;
    	newBounds.width = oldBounds.width;
    	newBounds.height = oldBounds.height-1;               	
    	footerLabel.setBounds(newBounds);
    	footerLabel.setBounds(oldBounds);
    	
    	if (message_timer == null) {
    		message_timer = new Timer(MESSAGE_DELAY, new ActionListener() {
    			public void actionPerformed(ActionEvent ev) {
    				footerLabel.setText(READY);
    				message_timer.stop();
    			}
    		});
    	}
    	if (message_timer.isRunning()) {
    		message_timer.restart();
    	} else {
    		message_timer.start();
    	}
    	
    }
    
    
	/**
	 * @return Returns the frame.
	 */
	public synchronized JFrame getFrame() {
		return frame;
	}
}
