/*
 * INRIA-OADYMPPAC
 * 
 * Ownership: Copyright (c) 1990-2004 COSYTEC S.A.
 * Author	: 	raf
 *           	COSYTEC
 *		    	4 rue Jean Rostand
 * 				91893 Orsay Cedex, France
 *				Tel. +33 1 60 19 37 38
 *				Fax. +33 1 60 19 36 20
 *
 * Purpose	: COMPLETE HERE
 * 
 * Created on 16 mars 2004
 *
 * 
 */
package fr.inria.oadymppac.parsing.event;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;



/**
 * @author raf
 *
 * @
 */
public abstract class EventListenerManagerImpl implements  EventListenerManager {
     
    protected Map listenersEventMap;
    
    protected Collection listenersList;
    
    /**
     * Passes back the event listener list as an array
     * of ListenerType-listener pairs.  Note that for 
     * performance reasons, this implementation passes back 
     * the actual data structure in which the listener data
     * is stored internally!  
     * This method is guaranteed to pass back a non-null
     * array, so that no null-checking is required in 
     * fire methods.  A zero-length array of Object should
     * be returned if there are currently no listeners.
     * 
     * WARNING!!! Absolutely NO modification of
     * the data contained in this array should be made -- if
     * any such manipulation is necessary, it should be done
     * on a copy of the array returned rather than the array 
     * itself.
     */
    public EventListener[] getListenerList() {
    	Object[] listeners = (listenersList == null ? null : listenersList.toArray());
    	EventListener[] result = (listeners == null ? null : new EventListener[listeners.length]);
    	if (listeners != null) {
    		for(int i=0; i < listeners.length; i++) {
    			result[i] = (EventListener) listeners[i];
    		}
    	}
        return (result==null ? NULL_ARRAY : result);   
    }
    
    /**
     * Return an array of all the listeners of the given type. 
     * @return all of the listeners of the specified type. 
     * @exception  ClassCastException if the supplied class
     *		is not assignable to EventListener
     * 
     * @since 1.3
     */
    public EventListener[] getListeners(Event e) {
        Class t =e.getClass();
        return getListeners(t.getName());
    }
    
    public EventListener[] getListeners(Class e) {
        return getListeners(e.getName());
    }
    
    public EventListener[] getListeners(String e) {
    	Collection listeners = (listenersEventMap == null ? null : (Collection) listenersEventMap.get(e));
    	EventListener[] result = (listeners == null ? null : new EventListener[listeners.size()]);
    	Iterator it = (listeners == null ? null : listeners.iterator());
    	int i=0;
    	while(it != null && it.hasNext() ) {
    			result[i++] = (EventListener) it.next();
    		}
        return (result == null ? NULL_ARRAY : result); 
    }
    
    /**
     * Returns the total number of listeners for this listener list.
     */
    public int getListenerCount() {
        return (listenersList == null ? 0 : listenersList.size());
    }
    
    /**
     * Returns the total number of listeners of the supplied type 
     * for this listener list.
     */
    public int getListenerCount(Event e) {
        Class t = e.getClass();
        return getListenerCount(t.getName());
    }
    public int getListenerCount(Class e) {
        return getListenerCount(e.getName());           
    }
    
    public int getListenerCount(String e) {
    	Collection listeners = (listenersEventMap == null ? null : (Collection) listenersEventMap.get(e));
    	return (listeners == null ? 0 : listeners.size());   
    }
    
 
    
    /**
     * Adds the listener as a listener of the specified type.
     * @param t the type of the listener to be added
     * @param l the listener to be added
     */
    public synchronized void addListener(Event e, EventListener l) {
        if (!Event.class.isInstance(e)) {
            throw new IllegalArgumentException("Parameter " + e +
            " is not an Event" );
        } else {
            Class t = e.getClass();
            addListener(t.getName(),l);
        }
    }
    
    public synchronized void addListener(Class e, EventListener l) {
        if (!Event.class.isAssignableFrom(e)) {
            throw new IllegalArgumentException("Parameter " + e +
            " is not an Event class" );
        } else {
            addListener(e.getName(),l);
        }
    }
    
    public synchronized void addListener(String e, EventListener l) {
        if (logger.isDebugEnabled()) logger.info("Adding new Listener "+l+" of Event "+e);
        if (l==null) {
            // In an ideal world, we would do an assertion here
            // to help developers know they are probably doing
            // something wrong
            return;
        } 
        buildListenersMap();
        Collection listeners = (Collection) listenersEventMap.get(e);
        if (listeners == null) {
        	listeners=new Vector();
        	listeners.add(l);
        	listenersEventMap.put(e,listeners);
        } else if (!listeners.contains(l)) {
        	listeners.add(l);
        }
        if (!listenersList.contains(l)) {
        	listenersList.add(l);
        }
        if (logger.isDebugEnabled()) logger.info("Added "+this.toString());
    }
    
    protected void buildListenersMap() {
    	if (listenersEventMap == null) listenersEventMap = new Hashtable();
    	if (listenersList == null) listenersList = new Vector();
    }
    
    
    /**
     * Removes the listener as a listener of the specified type.
     * @param t the type of the listener to be removed
     * @param l the listener to be removed
     */
    
    public synchronized void removeListener(Event e, EventListener l) {
        if (!Event.class.isInstance(e)) {
            throw new IllegalArgumentException("Parameter " + e +
            " is not an Event" );
        } else {
            Class t = e.getClass();
            removeListener(t.getName(),l);
        }
    }
    
    public synchronized void removeListener(Class e, EventListener l) {
        if (!Event.class.isAssignableFrom(e)) {
            throw new IllegalArgumentException("Parameter " + e +
            " is not an Event class" );
        } else {
            removeListener(e.getName(),l);
        }
    }
    
    public synchronized void removeListener(String e, EventListener l) {
        if (l ==null) {
            // In an ideal world, we would do an assertion here
            // to help developers know they are probably doing
            // something wrong
            return;
        }
        Collection listeners = (Collection) listenersEventMap.get(e);
        if (listeners != null ) {
        	listeners.remove(l);
        }
    }
    
    
    
    public void fireEvent(Event e) {
        EventListener[] lList = getListeners(e); // not a very efficient implementation !
        if(logger.isDebugEnabled()) logger.info("Must notify"+lList.length+" "+lList);
        for (int i = lList.length-1; i>=0; i-=1) {
            if(logger.isDebugEnabled()) logger.info("Firing event "+e+" to listener "+lList[i]);
            lList[i].eventProcess(e); 
        }
    }
    
    
}
