/*
 * INRIA-OADYMPPAC
 * 
 * Ownership: Copyright (c) 1990-2004 COSYTEC S.A.
 * Author	: 	raf
 *           	COSYTEC
 *		    	4 rue Jean Rostand
 * 				91893 Orsay Cedex, France
 *				Tel. +33 1 60 19 37 38
 *				Fax. +33 1 60 19 36 20
 *
 * Purpose	: COMPLETE HERE
 * 
 * Created on 11 mars 2004
 *
 * 
 */
package fr.inria.oadymppac.parsing.gentra4cp;

import java.util.StringTokenizer;

import org.xml.sax.Attributes;

import fr.inria.oadymppac.parsing.ContentHandler;
import fr.inria.oadymppac.parsing.event.Event;
import fr.inria.oadymppac.parsing.event.EventImpl;
import fr.inria.oadymppac.parsing.event.Info;
import fr.inria.oadymppac.parsing.gentra4cp.event.UpdateEvent;
import fr.inria.oadymppac.parsing.info.UpdateInfo;


/**
 * @author raf
 *
 * @
 */
public class BasicUpdateEvent extends EventImpl implements UpdateEvent {
    
    protected UpdateInfo info;
    private String token; 
    
    private StringTokenizer tokenizer;
    /**
     * 
     */
    public BasicUpdateEvent() {
        super();
        buildInfo();
    }

    protected void buildInfo() {
		if (info==null) info = new UpdateInfo();
	}
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.EventImpl#clone()
     */
    public Object clone() throws CloneNotSupportedException {
        return info.clone();
    }
 
	

    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#reset()
     */    
    protected short[] extractType(Attributes atts) {
        Event.logger.info("Extracting update type");
        buffer = atts.getValue(EVENT_TYPE);
        int i;
        short[] result;
        if (buffer != null) {
            tokenizer=new StringTokenizer(buffer);
            i=tokenizer.countTokens();
            if (i >0 ) {
                result= new short[i];
                i=0;
            } else {
                return TYPE_NONE;
            }
            while(tokenizer.hasMoreTokens()) {
                token = tokenizer.nextToken();
                if (token.equals(GROUND)) {
                    Event.logger.info("Ground update detected !");
                    result[i++]=TYPE_GROUND;
                } else if (token.equals(MIN)) {
                    Event.logger.info("Min update detected !");
                    result[i++]=TYPE_MIN;
                } else if (token.equals(MAX)) {
                    Event.logger.info("Max update detected !");
                    result[i++]=TYPE_MAX;
                } else if (token.equals(MINMAX)) {
                    Event.logger.info("Minmax update detected !");
                    result[i++]=TYPE_MINMAX;
                } else if (token.equals(VALUE)) {
                    Event.logger.info(" VALUE detected !");
                    result[i++]=TYPE_VALUE;
                } else if (token.equals(ANY)) {
                    Event.logger.info(" Any update detected !");
                    result[i++]=TYPE_ANY;
                } else if (token.equals(EMPTY)) {
                    Event.logger.info(" Empty update detected !");
                    result[i++]=TYPE_EMPTY;
                } else if (token.equals(NOTHING)) {
                    Event.logger.info(" Nothing update detected !");
                    result[i++]=TYPE_NOTHING;
                } else {
                    Event.logger.warn("Impossible to detect variable type!");
                }
            }
            return result;
        } else {
            return TYPE_NONE;
        }
    }
    /**
     * @return Returns the info.
     */
    public Info getInfo() {
        return (Info)info;
    } 
    /**
     * @return
     */
    public short[] getType() {
        return info.getType();
    }
    /**
     * @return
     */
    public String getVariable() {
        return info.getVariable();
    }
    
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#init(fr.inria.oadymppac.ContentHandler, org.xml.sax.Attributes)
     */
    public void init(ContentHandler handler, Attributes atts) {
    	buildInfo();
        super.init(handler, atts);
        setVariable(extractString(VARIABLE,atts, VARIABLE_NONE));
        setVariable(extractString(STATUS,atts, STATUS_NONE));
        setType(extractType(atts));
    }
    
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#reset()
     */
    public void reset() {
    	super.reset();
    	if (info != null) {
        setVariable(VARIABLE_NONE);
        info=null;
    	}
    }
    /**
     * @param type
     */
    public void setType(short[] type) {
        info.setType(type);
    }
    /**
     * @param var
     */
    public void setVariable(String var) {
        info.setVariable(var);
    }
    
    public int getEventType() {
		return EVENT_TYPE;
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.parsing.gentra4cp.event.attributes.UpdateAttributes#getStatus()
	 */
	public String getStatus() {
		return info.getStatus();
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.parsing.gentra4cp.event.attributes.UpdateAttributes#setStatus(java.lang.String)
	 */
	public void setStatus(String status) {
		info.setStatus(status);
	}
}