/*
 * INRIA-OADYMPPAC
 * 
 * Ownership: Copyright (c) 1990-2004 COSYTEC S.A.
 * Author	: 	raf
 *           	COSYTEC
 *		    	4 rue Jean Rostand
 * 				91893 Orsay Cedex, France
 *				Tel. +33 1 60 19 37 38
 *				Fax. +33 1 60 19 36 20
 *
 * Purpose	: COMPLETE HERE
 * 
 * Created on 11 mars 2004
 *
 * 
 */
package fr.inria.oadymppac.parsing.gentra4cp;

import org.xml.sax.Attributes;

import fr.inria.oadymppac.parsing.ContentHandler;
import fr.inria.oadymppac.parsing.event.Event;
import fr.inria.oadymppac.parsing.event.EventImpl;
import fr.inria.oadymppac.parsing.event.Info;
import fr.inria.oadymppac.parsing.gentra4cp.event.VariableEvent;
import fr.inria.oadymppac.parsing.info.VariableInfo;


/**
 * @author raf
 *
 * @
 */
public class BasicVariableEvent extends EventImpl implements VariableEvent {
    
    private String buffer;
    
    protected VariableInfo info;
    /**
     * 
     */
    public BasicVariableEvent() {
        super();
        buildInfo();
    }
    protected void buildInfo() {
		if (info==null) info = new VariableInfo();
	}
    
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#reset()
     */    
    protected short extractType(Attributes atts) {
        Event.logger.info("Extracting variable type");
        buffer = atts.getValue(EVENT_TYPE);
        if (buffer != null) {
            buffer = buffer.intern();
            if (buffer.equals(INT)) {
                Event.logger.info("Integer Domain variable detected !");
                return TYPE_INT;
            } else if (buffer.equals(RAT)) {
                Event.logger.info("Rational variable detected !");
                return TYPE_RAT;
            } else if (buffer.equals(REAL)) {
                Event.logger.info("Real variable detected !");
                return TYPE_REAL;
            } else if (buffer.equals(ENUM)) {
                Event.logger.info("Enumeration variable detected !");
                return TYPE_ENUM;
            } else if (buffer.equals(STRING)) {
                Event.logger.info(" String variable detected !");
                return TYPE_RAT;
            } else {
                Event.logger.warn("Impossible to detect variable type!");
                return TYPE_NONE;
            }
        } else {
            Event.logger.warn("No variable type provided!");
            return TYPE_NONE;
        }
    }

    /**
     * @return
     */
    public String getExternal() {
        return info.getExternal();
    }
    /**
     * @return
     */
    public String getIdentifier() {
        return info.getIdentifier();
    }
    /**
     * @return Returns the info.
     */
    public Info getInfo() {
        return (Info)info;
    }
    /**
     * @return
     */
    public String getInternal() {
        return info.getInternal();
    }
    /**
     * @return
     */
    public String getName() {
        return info.getName();
    }
    /**
     * @return
     */
    public short getType() {
        return info.getType();
    }
    
    
    
   
    
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#init(fr.inria.oadymppac.ContentHandler, org.xml.sax.Attributes)
     */
    public void init(ContentHandler handler, Attributes atts) {
    	buildInfo();
        super.init(handler, atts);
        setIdentifier(extractString(IDENTIFIER,atts, IDENTIFIER_NONE));
        setName(extractString(NAME,atts, NAME_NONE));
        setInternal(extractString(INTERNAL,atts, INTERNAL_NONE));
        setExternal(extractString(EXTERNAL,atts, EXTERNAL_NONE));
        setType(extractType(atts));
    }
    
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#reset()
     */
    public void reset() {
    	super.reset();
    	if (info != null) {
        setIdentifier(IDENTIFIER_NONE);
        setName(NAME_NONE);
        setInternal(INTERNAL_NONE);
        setExternal(EXTERNAL_NONE);
        setType(TYPE_NONE);
        info=null;
    	}
    }
    /**
     * @param external
     */
    public void setExternal(String external) {
        info.setExternal(external);
    }
    /**
     * @param ident
     */
    public void setIdentifier(String ident) {
        info.setIdentifier(ident);
    }
    /**
     * @param internal
     */
    public void setInternal(String internal) {
        info.setInternal(internal);
    }
    /**
     * @param name
     */
    public void setName(String name) {
        info.setName(name);
    }
    /**
     * @param type
     */
    public void setType(short type) {
        info.setType(type);
    }
    
    public int getEventType() {
		return EVENT_TYPE;
	}
}