/*
 * $Id: ExceptionManager.java,v 1.5 2004/05/28 06:28:47 raf Exp $
 *
 * Copyright (c) 2001-2002 Jean-Daniel Fekete, Mohammad Ghoniem and
 * Ecole des Mines de Nantes.  All rights reserved.
 *
 * This software is proprietary information of Jean-Daniel Fekete and
 * Ecole des Mines de Nantes.  You shall use it only in accordance
 * with the terms of the license agreement you accepted when
 * downloading this software.  The license is available in the file
 * licence.txt and at the following URL:
 * http://www.emn.fr/fekete/oadymppac/licence.html
 */
package fr.inria.oadymppac.solver;

import java.util.Iterator;
import java.util.Vector;

import fr.inria.oadymppac.utils.*;

/**
 * A <code>VariableManager</code> is a utility class that manages the
 * domain of all the variables declared and used during a session.  It
 * keeps track of the domains and notifies listeners when the domains
 * change.
 *
 * @author Jean-Daniel Fekete
 * @author Mohammad Ghoniem
 * @version $Revision: 1.5 $
 */
public class ExceptionManager extends ObjectManagerImpl implements SolverManagable, Indexable{
	public static final String PROPERTY_NAME = "EXCEPTION_MANAGER";
    

	Solver solver;
	
    /**
     * Also keep the variables in the order they have been defined.
     */
    Vector exceptions;
    
    

    
    /**
     * protected constructor.
     */
    public ExceptionManager() {
		exceptions = new Vector();
    }

 

 
    
    
    /**
     * Returns the <code>index</code>'s created variable.
     */
    public synchronized Exception get(int index) {
		return (Exception)exceptions.elementAt(index);
    }
    
    

    /**
     * Return the number of variables in the manager.
     */
    public synchronized int getCount() {
		return exceptions.size();
    }

    

    /**
     * Return the iterator on all the variables.
     */
    public synchronized Iterator iterator() {
		return exceptions.iterator();
    }

 

    public synchronized static ExceptionManager getExceptionManager(Solver solver) {
    	return (ExceptionManager) getObjectManager(PROPERTY_NAME, ExceptionManager.class, solver);
    }

    /**
     * When a <code>NewVariableEvent</code> is received, the corresponding
     * variable is added to the list of variables.
     */
    public synchronized int add(Exception exception) {
    	exception.printStackTrace();
    	return (exceptions.add(exception) ? exceptions.indexOf(exception) : INVALID);
    }
	
	
	/**
	 * @return Returns the solver.
	 */
	public synchronized Solver getSolver() {
		return solver;
	}
	/**
	 * @param solver The solver to set.
	 */
	public synchronized void setSolver(Solver solver) {
		this.solver = solver;
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.Identifiable#getIndex()
	 */
	public int getIndex() {
		return Solver.EXCEPTION;
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.Identifiable#getKey()
	 */
	public Object getKey() {
		return PROPERTY_NAME;
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.Identifiable#setIndex(int)
	 */
	public void setIndex(int index) {
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.ObjectManager#getFromIndex(int)
	 */
	public Object getFromIndex(int index) {
		return exceptions.elementAt(index);
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.ObjectManager#getFromKey(java.lang.Object)
	 */
	public Object getFromKey(Object key) {
		return key;
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.ObjectManager#getIndex(java.lang.Object)
	 */
	public int getIndex(Object key) {
		return exceptions.indexOf(key);
	}
	/**
	 * @param o
	 * @return
	 */
	public int add(Object o) {
		return (exceptions.add(o) ? exceptions.indexOf(o) : INVALID);
	}
	
	public void reset() {
		owner.setProperty(PROPERTY_NAME, null);
	}
}
