/*
 * $Id: RestoreManager.java,v 1.5 2004/05/28 06:28:47 raf Exp $
 *
 * Copyright (c) 2001-2002 Jean-Daniel Fekete, Mohammad Ghoniem and
 * Ecole des Mines de Nantes.  All rights reserved.
 *
 * This software is proprietary information of Jean-Daniel Fekete and
 * Ecole des Mines de Nantes.  You shall use it only in accordance
 * with the terms of the license agreement you accepted when
 * downloading this software.  The license is available in the file
 * licence.txt and at the following URL:
 * http://www.emn.fr/fekete/oadymppac/licence.html
 */
package fr.inria.oadymppac.solver;

import java.util.Hashtable;
import java.util.Map;

import fr.inria.oadymppac.utils.Indexable;

/**
 * A <code>RestoreManager</code> is a utility class that manages the
 * domain of all the variables declared and used during a session.  It
 * keeps track of the domains and notifies listeners when the domains
 * change.
 *
 * @author Jean-Daniel Fekete
 * @author Mohammad Ghoniem
 * @version $Revision: 1.5 $
 */
public final class RestoreManager extends SolverObjectManager implements Indexable  {
	private static final String PROPERTY_NAME = "RESTORE_MANAGER";
    
    
    
    /**
     * Returns the <code>index</code>'s created variable.
     */
    public synchronized Restore get(int index) {
		return (Restore)getFromIndex(index);
    }
    
    /**
     * Returns the <code>index</code>'s created variable.
     */
    public synchronized Restore get(Object key) {
		return (Restore) getFromKey(key);
    }

   

    public static RestoreManager getRestoreManager(Solver solver) {
	return (RestoreManager) getObjectManager(PROPERTY_NAME,RestoreManager.class, solver );
    }

    
    
	
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.ObjectManager#buildIndexer()
	 */
	protected Map buildIndexer() {
		return new Hashtable();
	}
	
	
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.Identifiable#getIndex()
	 */
	public int getIndex() {
		return Solver.RESTORE;
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.Identifiable#getKey()
	 */
	public Object getKey() {
		return PROPERTY_NAME;
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.Identifiable#setIndex(int)
	 */
	public void setIndex(int index) {
	}
	
	public void reset() {
		owner.setProperty(PROPERTY_NAME, null);
	}
}
