/*
 * INRIA-OADYMPPAC
 * 
 * Ownership: Copyright (c) 1990-2004 COSYTEC S.A.
 * Author	: 	raf
 *           	COSYTEC
 *		    	4 rue Jean Rostand
 * 				91893 Orsay Cedex, France
 *				Tel. +33 1 60 19 37 38
 *				Fax. +33 1 60 19 36 20
 *
 * Purpose	: COMPLETE HERE
 * 
 * Created on 17 mars 2004
 *
 * 
 */
package fr.inria.oadymppac.solver.parsing;

import infovis.Graph;

import org.apache.log4j.Logger;

import fr.inria.oadymppac.parsing.gentra4cp.event.AwakeEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.NewConstraintEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.ReduceEvent;
import fr.inria.oadymppac.parsing.info.AwakeInfo;
import fr.inria.oadymppac.parsing.info.NewConstraintInfo;
import fr.inria.oadymppac.parsing.info.ReduceInfo;
import fr.inria.oadymppac.solver.Awake;
import fr.inria.oadymppac.solver.AwakeManager;
import fr.inria.oadymppac.solver.Constraint;
import fr.inria.oadymppac.solver.ConstraintManager;
import fr.inria.oadymppac.solver.Reduce;
import fr.inria.oadymppac.solver.ReduceManager;
import fr.inria.oadymppac.solver.Solver;
import fr.inria.oadymppac.solver.analyser.ConstraintConstraintGraphAnalyser;

/**
 * @author raf
 *  @
 */
public class ConstraintConstraintManagerEventListener extends AbstractManagerEventListener { 
	public static Logger logger = Logger.getLogger(ConstraintConstraintManagerEventListener.class);
	
	ConstraintConstraintGraphAnalyser analyser;
	
	private boolean performEndAnalyze=true;

 
    public void eventProcess(NewConstraintEvent ev) {
    	super.eventProcess(ev);
    	try {    		
    		NewConstraintInfo info = (NewConstraintInfo) ev.getInfo();
    		int ctr=solver.get(Solver.CONSTRAINT, ConstraintManager.key(extractConstraintIdentifier(info.getIdentifier())));
    		if (ctr != ConstraintManager.INVALID) {
    			analyser.add((Constraint)solver.get(Solver.CONSTRAINT, ctr));
     		} else {
   				if (logger.isDebugEnabled()) logger.warn("Impossible to get corresponding constraint "+info.getIdentifier());
   			}
    	} catch (Exception exp) {
    		solver.add(Solver.EXCEPTION,exp);
    	}
        
    }

   
    public void eventProcess(ReduceEvent ev) {
    	super.eventProcess(ev);
    	try {
    		
    		ReduceInfo info = (ReduceInfo) ev.getInfo();
    		int red=solver.get(Solver.REDUCE,ReduceManager.key(extractEventIdentifier(REDUCE,info.getChrono())));
    		if (red != ReduceManager.INVALID) {
    			Reduce reduce = (Reduce)solver.get(Solver.REDUCE,red);
    			if (reduce != null) analyser.add(reduce);
   			} else {
   				if (logger.isDebugEnabled())  {
   					logger.warn("Impossible to get corresponding reduce at chrono"+info.getChrono());
   					logger.warn("using default Reduce "+extractEventIdentifier(REDUCE,info.getChrono()) );
   				}
   			}
    	} catch (Exception exp) {
    		solver.add(Solver.EXCEPTION, exp);
    	}
    }
    
    public void eventProcess(AwakeEvent ev) {
    	super.eventProcess(ev);
    	try {
    		
    		AwakeInfo info = (AwakeInfo) ev.getInfo();
    		int awk=solver.get(Solver.AWAKE,AwakeManager.key(extractEventIdentifier(AWAKE,info.getChrono())));
    		if (awk != AwakeManager.INVALID) {
    			Awake awake =  (Awake)solver.get(Solver.AWAKE,awk);
    			if (awake != null) analyser.add(awake);
   			} else {
   				if (logger.isDebugEnabled())  {
   					logger.warn("Impossible to get corresponding awake at chrono"+info.getChrono());
   					logger.warn("using default Awake "+extractEventIdentifier(AWAKE,info.getChrono()) );
   				}
   			}
    	} catch (Exception exp) {
    		solver.add(Solver.EXCEPTION, exp);
    	}
    }
    
	/**
	 * @return Returns the solver.
	 */
	public Solver getSolver() {
		return solver;
	}
	/**
	 * @param solver The solver to set.
	 */
	public void setSolver(Solver solver) {
		this.solver = solver;
	}
	/**
	 * 
	 */
	public ConstraintConstraintManagerEventListener(Solver solver) {
		super(solver);
		setAnalyser(new ConstraintConstraintGraphAnalyser(solver));
		
	}
	/**
	 * @return Returns the graph.
	 */
	public Graph getGraph() {
		return analyser.getGraph();
	}

	/**
	 * @return Returns the analyser.
	 */
	public synchronized ConstraintConstraintGraphAnalyser getAnalyser() {
		return analyser;
	}
	/**
	 * @param analyser The analyser to set.
	 */
	public synchronized void setAnalyser(
			ConstraintConstraintGraphAnalyser analyser) {
		this.analyser = analyser;
	}
	
	
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.parsing.event.EventListener#processEnd()
	 */
	public void processStart() {
		performEndAnalyze=true;
	};
	
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.parsing.event.EventListener#processEnd()
	 */
	public void processEnd() {
		if (analyser != null && performEndAnalyze) {
			analyser.endAnalyse();
			performEndAnalyze=false;
		}
	}
	
}
