/*
 * INRIA-OADYMPPAC
 * 
 * Ownership: Copyright (c) 1990-2004 COSYTEC S.A.
 * Author	: 	raf
 *           	COSYTEC
 *		    	4 rue Jean Rostand
 * 				91893 Orsay Cedex, France
 *				Tel. +33 1 60 19 37 38
 *				Fax. +33 1 60 19 36 20
 *
 * Purpose	: COMPLETE HERE
 * 
 * Created on 17 mars 2004
 *
 * 
 */
package fr.inria.oadymppac.solver.parsing;

import fr.inria.oadymppac.application.ApplicationProperties;
import fr.inria.oadymppac.panel.MainFrameDecorator;
import fr.inria.oadymppac.parsing.gentra4cp.event.AwakeEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.NewConstraintEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.NewVariableEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.PostEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.ReduceEvent;
import fr.inria.oadymppac.parsing.info.AwakeInfo;
import fr.inria.oadymppac.parsing.info.EventInfo;
import fr.inria.oadymppac.parsing.info.NewConstraintInfo;
import fr.inria.oadymppac.parsing.info.NewVariableInfo;
import fr.inria.oadymppac.parsing.info.PostInfo;
import fr.inria.oadymppac.parsing.info.ReduceInfo;
import fr.inria.oadymppac.solver.Awake;
import fr.inria.oadymppac.solver.Constraint;
import fr.inria.oadymppac.solver.DefaultAwake;
import fr.inria.oadymppac.solver.DefaultConstraint;
import fr.inria.oadymppac.solver.DefaultPost;
import fr.inria.oadymppac.solver.DefaultReduce;
import fr.inria.oadymppac.solver.DefaultVariable;
import fr.inria.oadymppac.solver.Post;
import fr.inria.oadymppac.solver.Reduce;
import fr.inria.oadymppac.solver.Solver;
import fr.inria.oadymppac.solver.SolverException;
import fr.inria.oadymppac.solver.Variable;
import fr.inria.oadymppac.solver.parsing.adapter.AwakeAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.CauseAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.ConstraintAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.ConstraintStatusAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultAwakeAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultCauseAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultConstraintAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultConstraintStatusAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultDeltaAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultDomainAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultExplanationAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultPostAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultReduceAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultSearchNodeAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultStateAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultUpdateAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultValueListAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultVariableAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DefaultVariableStatusAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DeltaAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.DomainAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.ExplanationAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.PostAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.ReduceAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.SearchNodeAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.SolverAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.StateAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.UpdateAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.ValueListAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.VariableAdapter;
import fr.inria.oadymppac.solver.parsing.adapter.VariableStatusAdapter;

/**
 * @author raf
 *  @
 */
public class SolverManagerEventListener extends AbstractManagerEventListener {
	protected final static String NONE = SolverAdapter.NONE;
	
	
	private String buffer;
	
	private MainFrameDecorator frame_deco;
	
	private int current_chrono =0;

	
	private int chrono_min;
	private int chrono_max;
	
	
	private int current_depth =0;

	private int depth_min;
	private int depth_max;
    
	
	private VariableAdapter variableAdapter;
    private ConstraintAdapter constraintAdapter; 
    private SearchNodeAdapter searchNodeAdapter;
    private ReduceAdapter reduceAdapter;
    private ValueListAdapter valueListAdapter;
    private DomainAdapter domainAdapter;
    private DeltaAdapter deltaAdapter;
    private VariableStatusAdapter variableStatusAdapter;
    private ConstraintStatusAdapter constraintStatusAdapter;
    private UpdateAdapter updateAdapter;
    private StateAdapter stateAdapter;
    private PostAdapter postAdapter;
    private AwakeAdapter awakeAdapter;
    private ExplanationAdapter explanationAdapter;
    private CauseAdapter causeAdapter;
    
    
    protected void updateSolver(EventInfo info) {
    	
    	current_chrono = info.getChrono();
	    chrono_max =(chrono_max > current_chrono ? chrono_max : current_chrono) ;
		chrono_min=(chrono_min < current_chrono ? chrono_min : current_chrono) ;
		
	    current_depth = info.getDepth();
	    depth_max =(depth_max > current_depth ? depth_max : current_depth) ;
		depth_min=(depth_min < current_depth ? depth_min : current_depth) ;
		solver.setIntValue(Solver.CURRENT_CHRONO, current_chrono);
		solver.setIntValue(Solver.CURRENT_DEPTH, current_depth);
		MainFrameDecorator deco = (MainFrameDecorator) solver.getProperty(ApplicationProperties.APPLICATION_FRAME_DECORATOR);
		deco.setMessage("Chrono:"+current_chrono);
    }

    public void eventProcess(NewVariableEvent ev) {
    	super.eventProcess(ev);
    	try {
    		NewVariableInfo info = (NewVariableInfo) ev.getInfo();
    		updateSolver(info);	
    		Variable variable= new DefaultVariable(this.getSolver());
    		info.setIdentifier(extractVariableIdentifier(info.getIdentifier()));
    		if (variableAdapter != null) variableAdapter.adapt(info, variable);
    	} catch (SolverException exp) {
    		solver.add(Solver.EXCEPTION, exp);
    	} catch (CloneNotSupportedException exp) {
    		solver.add(Solver.EXCEPTION, exp);
		}
        
    }

    public void eventProcess(NewConstraintEvent ev) {
    	super.eventProcess(ev);
       	try {
       		NewConstraintInfo info = (NewConstraintInfo) ev.getInfo();
       		updateSolver(info);	
    		Constraint constraint= new DefaultConstraint(this.getSolver());
    		constraint.setIdentifier((extractConstraintIdentifier(info.getIdentifier())));
    		if (constraintAdapter != null) constraintAdapter.adapt(info, constraint);
    				
    	} catch (Exception exp) {
    		solver.add(Solver.EXCEPTION,exp);
    	}

    }

    public void eventProcess(PostEvent ev) {
    	super.eventProcess(ev);
    	try {
			PostInfo info = (PostInfo) ev.getInfo();
			updateSolver(info);
			Post post = new DefaultPost(this.getSolver());
    		info.setConstraint(extractConstraintIdentifier(info.getConstraint()));
    		if (postAdapter != null) postAdapter.adapt(info, post);
    			
		} catch (CloneNotSupportedException e) {
			solver.add(Solver.EXCEPTION,e);
		} catch (SolverException e) {
			solver.add(Solver.EXCEPTION,e);
		}
    }
/*
    public void eventProcess(ChoicePointEvent ev) {
    	super.eventProcess(ev);	
    	try {
    		ChoicePointInfo info = (ChoicePointInfo) ev.getInfo();
    		ChoicePoint choicePoint= new DefaultChoicePoint(this.getSolver());
    		
    		choicePoint.setIdentifier((extractChoicePointIdentifier(info.getNode())));
    		if (searchNodeAdapter != null) searchNodeAdapter.adapt(info, choicePoint);
    		buffer = info.getNodeName(); 
    		buffer = (buffer==ChoicePointInfo.NODENAME_NONE ? choicePoint.getIdentifier() : buffer);
    		
    		choicePoint.setName(buffer);
    		int cp=solver.add(Solver.SEARCHNODE,choicePoint);	
    		updateSolver(info);
    	} catch (Exception exp) {
    		solver.add(Solver.EXCEPTION,exp);
    	}
    }

    public void eventProcess(SolutionEvent ev) {
    	super.eventProcess(ev);	
    	try {
    		SolutionInfo info = (SolutionInfo) ev.getInfo();
    		Solution solution= new DefaultSolution(this.getSolver());
    		
    		solution.setIdentifier((extractSolutionIdentifier(info.getNode())));
    		searchNodeAdapter.adapt(info, solution);
    		buffer = info.getNodeName(); 
    		buffer = (buffer==SolutionInfo.NODENAME_NONE ? solution.getIdentifier() : buffer);
    		
    		solution.setName(buffer);
    		int sol=solver.add(Solver.SEARCHNODE,solution);	
    		updateSolver(info);
    	} catch (Exception exp) {
    		solver.add(Solver.EXCEPTION,exp);
    	}
    }

    public void eventProcess(FailureEvent ev) {
    	super.eventProcess(ev);
    	try {
    		FailureInfo info = (FailureInfo) ev.getInfo();
    		Failure failure= new DefaultFailure(this.getSolver());
    		
    		failure.setIdentifier((extractFailureIdentifier(info.getNode())));
    		searchNodeAdapter.adapt(info, failure);
    		buffer = info.getNodeName(); 
    		buffer = (buffer==ChoicePointInfo.NODENAME_NONE ? info.getNode() : buffer);
    		
    		info.setNodeName(buffer);
    		int cp=solver.add(Solver.SEARCHNODE,failure);	
    		updateSolver(info);
    	} catch (Exception exp) {
    		solver.add(Solver.EXCEPTION,exp);
    	}
    }
/*
    public void eventProcess(BackToEvent ev) {
    super.eventProcess(ev);
    	if (logger.isDebugEnabled()) logger.info("------------------ BACK TO EVENT !!! ------------------");
        if (logger.isDebugEnabled()) logger.info("CHRONO:" + ev.getChrono());
        if (logger.isDebugEnabled()) logger.info("DEPTH:" + ev.getDepth());
        if (logger.isDebugEnabled()) logger.info("NODE:" + ev.getNode());
        if (logger.isDebugEnabled()) logger.info("NODE BEFORE:" + ev.getNodeBefore());
        showDeltas(ev);
        showState(ev);
    }
*/
    public void eventProcess(ReduceEvent ev) {
    	super.eventProcess(ev);
    	try {
    		ReduceInfo info = (ReduceInfo) ev.getInfo();
    		updateSolver(info);
    		Reduce reduce= new DefaultReduce(this.getSolver());
    		info.setChrono(extractEventIdentifier(REDUCE, info.getChrono()));
    		info.setVariable(extractVariableIdentifier(info.getVariable()));
    		info.setConstraint(extractConstraintIdentifier(info.getConstraint()));
    		if (reduceAdapter != null) reduceAdapter.adapt(info, reduce);
    		
    	} catch (Exception exp) {
    		solver.add(Solver.EXCEPTION,exp);
    	}
    }

    public void eventProcess(AwakeEvent ev) {
    super.eventProcess(ev);
  
    	try {
			AwakeInfo info = (AwakeInfo) ev.getInfo();
			updateSolver(info);
			Awake awake = new DefaultAwake(this.getSolver());
    		info.setConstraint(extractConstraintIdentifier(info.getConstraint()));
    		if (awakeAdapter != null) awakeAdapter.adapt(info, awake);
		} catch (CloneNotSupportedException e) {
			solver.add(Solver.EXCEPTION,e);
		} catch (SolverException e) {
			solver.add(Solver.EXCEPTION,e);
		}
    }
    
/*
    public void eventProcess(ScheduleEvent ev) {
    super.eventProcess(ev);
    	if (logger.isDebugEnabled()) logger.info("------------------ SCHEDULE EVENT !!! ------------------");
        if (logger.isDebugEnabled()) logger.info("CHRONO:" + ev.getChrono());
        if (logger.isDebugEnabled()) logger.info("DEPTH:" + ev.getDepth());
        if (logger.isDebugEnabled()) logger.info("CONSTRAINT:" + ev.getConstraint());
        showScheduleActions(ev.getActions());
        showUpdate(ev);
        showState(ev);
    }

    public void eventProcess(RemoveEvent ev) {
    super.eventProcess(ev);
    	if (logger.isDebugEnabled()) logger.info("------------------ REMOVE EVENT !!! ------------------");
        if (logger.isDebugEnabled()) logger.info("CHRONO:" + ev.getChrono());
        if (logger.isDebugEnabled()) logger.info("DEPTH:" + ev.getDepth());
        if (logger.isDebugEnabled()) logger.info("CONSTRAINT:" + ev.getConstraint());
        showState(ev);

    }

    public void eventProcess(RestoreEvent ev) {
    super.eventProcess(ev);
        if (logger.isDebugEnabled()) logger.info("------------------ RESTORE EVENT !!! ------------------");
        if (logger.isDebugEnabled()) logger.info("CHRONO:" + ev.getChrono());
        if (logger.isDebugEnabled()) logger.info("DEPTH:" + ev.getDepth());
        if (logger.isDebugEnabled()) logger.info("VARIABLE:" + ev.getVariable());
        showDeltas(ev);
        showState(ev);

    }

    public void eventProcess(SuspendEvent ev) {
    super.eventProcess(ev);
        if (logger.isDebugEnabled()) logger.info("------------------ SUSPEND EVENT !!! ------------------");
        if (logger.isDebugEnabled()) logger.info("CHRONO:" + ev.getChrono());
        if (logger.isDebugEnabled()) logger.info("DEPTH:" + ev.getDepth());
        if (logger.isDebugEnabled()) logger.info("CONSTRAINT:" + ev.getConstraint());
        showState(ev);

    }

    public void eventProcess(SolvedEvent ev) {
    super.eventProcess(ev);
        if (logger.isDebugEnabled()) logger.info("------------------ SOLVED EVENT !!! ------------------");
        if (logger.isDebugEnabled()) logger.info("CHRONO:" + ev.getChrono());
        if (logger.isDebugEnabled()) logger.info("DEPTH:" + ev.getDepth());
        if (logger.isDebugEnabled()) logger.info("CONSTRAINT:" + ev.getConstraint());
        showState(ev);

    }

    public void eventProcess(RejectEvent ev) {
    super.eventProcess(ev);
        if (logger.isDebugEnabled()) logger.info("------------------ REJECT EVENT !!! ------------------");
        if (logger.isDebugEnabled()) logger.info("CHRONO:" + ev.getChrono());
        if (logger.isDebugEnabled()) logger.info("DEPTH:" + ev.getDepth());
        if (logger.isDebugEnabled()) logger.info("CONSTRAINT:" + ev.getConstraint());
        showState(ev);

    }

    public void eventProcess(AnnotationEvent ev) {
    super.eventProcess(ev);
        if (logger.isDebugEnabled()) logger.info("------------------ ANNOTATION EVENT !!! ------------------");
        if (logger.isDebugEnabled()) logger.info("CHRONO:" + ev.getChrono());
        if (logger.isDebugEnabled()) logger.info("DEPTH:" + ev.getDepth());
        if (logger.isDebugEnabled()) logger.info("ANNOTATION:" + ev.getIdentifier());
        AnnotationInfo annotation;
        AnnotationCommandInfo annotationInfo;
        try {
            annotation = (AnnotationInfo) ev.getInfo();
            annotationInfo = (AnnotationCommandInfo) annotation.getCommand();
            if (annotationInfo != null)
                if (logger.isDebugEnabled()) logger.info("COMMAND:" + annotationInfo.getCommand());
            else
                if (logger.isDebugEnabled()) logger.info("NO COMMAND PROVIDED");
        } catch (CloneNotSupportedException excp) {
            excp.printStackTrace();
        }

    }

    public void eventProcess(StateEvent ev) {
    super.eventProcess(ev);
        if (logger.isDebugEnabled()) logger.info("------------------ COMPLEMENT STATE EVENT !!! ------------------");
        StateInfo info = null;
        try {
            info = (StateInfo) ev.getInfo();
        } catch (CloneNotSupportedException excp) {
            logger.error("State Doesnot containt info", excp);
        }
        if (info != null) showState(info);
    }

    protected void showValueList(Event ev) {
    super.eventProcess(ev);
        ValueListProvider valueList = (ValueListProvider) ev;
        showValueList(valueList.getValueList());
    }

    protected void showValueList(ValueListInfo dom) {
        Iterator it = (dom == null ? null : dom.iterator());
        if (logger.isDebugEnabled()) logger.info("DOMAIN:< ");
        while (it != null && it.hasNext())
            if (logger.isDebugEnabled()) logger.info((it.next()).toString() + " ");
        if (logger.isDebugEnabled()) logger.info(" >");
    }

    protected void showChoiceConstraints(Event ev) {
        ChoiceConstraintListProvider choicConstraintList = (ChoiceConstraintListProvider) ev;
        ChoiceConstraintListInfo choicConstraints = choicConstraintList
                .getChoiceConstraintList();
        if (choicConstraints != null) showChoiceConstraints(choicConstraints);
    }

    protected void showChoiceConstraints(
            ChoiceConstraintListInfo choicConstraints) {
        ChoiceConstraintInfo info;
        Iterator it = (choicConstraints == null ? null : choicConstraints
                .iterator());
        if (logger.isDebugEnabled()) logger.info("CHOICE CONSTRAINTS:< ");
        while (it != null && it.hasNext()) {
            info = (ChoiceConstraintInfo) it.next();
            if (logger.isDebugEnabled()) logger.info("\tVARIABLE:" + info.getVariable());
            if (logger.isDebugEnabled()) logger.info("\tVALUE:" + info.getValue());
        }
        if (logger.isDebugEnabled()) logger.info(" >");
    }

    protected void showDeltas(Event ev) {
        DeltaListProvider deltaList = (DeltaListProvider) ev;
        DeltaListInfo deltas = deltaList.getDeltaList();
        DeltaInfo info;
        Iterator it = (deltas == null ? null : deltas.iterator());
        if (logger.isDebugEnabled()) logger.info("UPDATES:< ");
        while (it != null && it.hasNext()) {
            info = (DeltaInfo) it.next();
            showDelta(info);
        }
        if (logger.isDebugEnabled()) logger.info(" >");
    }

    protected void showUpdates(Event ev) {
        UpdateListProvider updateList = (UpdateListProvider) ev;
        UpdateListInfo updates = updateList.getUpdateList();
        if (updates != null) showUpdates(updates);
    }

    protected void showUpdates(UpdateListInfo updates) {
        UpdateInfo info;
        Iterator it = (updates == null ? null : updates.iterator());
        if (logger.isDebugEnabled()) logger.info("UPDATES:< ");
        while (it != null && it.hasNext()) {
            info = (UpdateInfo) it.next();
            showUpdate(info);
        }
        if (logger.isDebugEnabled()) logger.info(" >");
    }

    protected void showUpdate(Event ev) {
        UpdateProvider update = (UpdateProvider) ev;
        UpdateInfo info = update.getUpdate();
        if (info != null) showUpdate(info);
    }

    protected void showUpdate(UpdateInfo info) {
        if (logger.isDebugEnabled()) logger.info("UPDATE VARIABLE:" + info.getVariable());
        showUpdateTypes(info.getType());
        ValueListInfo valueList = info.getValueList();
        if (valueList != null)
            showValueList(valueList);
        else
            if (logger.isDebugEnabled()) logger.info("No values provided for update");
    }

    protected void showVarDomain(Event ev) {
        VarDomainProvider vardomain = (VarDomainProvider) ev;
        VarDomainInfo info = vardomain.getVarDomain();
        if (info != null) showVarDomain(info);
    }

    protected void showVarDomain(VarDomainInfo info) {
        if (logger.isDebugEnabled()) logger.info("VARIABLE DOMAIN:");
        if (logger.isDebugEnabled()) logger.info("\tMIN:" + info.getMin());
        if (logger.isDebugEnabled()) logger.info("\tMAX:" + info.getMax());
        if (logger.isDebugEnabled()) logger.info("\tSIZE:" + info.getSize());
        ValueListInfo valueList = info.getValueList();
        if (valueList != null)
            showValueList(valueList);
        else
            if (logger.isDebugEnabled()) logger.info("No values provided for update");
    }

    protected void showDelta(Event ev) {
        DeltaProvider vardomain = (DeltaProvider) ev;
        DeltaInfo info = vardomain.getDelta();
        if (info != null) showDelta(info);
    }

    protected void showDelta(DeltaInfo info) {
        if (logger.isDebugEnabled()) logger.info("DELTA DOMAIN:" + info.getVariable());
        ValueListInfo valueList = info.getValueList();
        if (valueList != null)
            showValueList(valueList);
        else
            if (logger.isDebugEnabled()) logger.info("No values provided for update");
    }

    protected void showUpdateTypes(short[] types) {
        if (logger.isDebugEnabled()) logger.info("\tTYPES: < ");
        if (types != null && types.length > 0) {
            for (int i = 0; i < types.length - 1; i++)
                if (logger.isDebugEnabled()) logger.info("" + types[i] + ",");
            if (logger.isDebugEnabled()) logger.info("" + types[types.length - 1] + " >");
        } else {
            if (logger.isDebugEnabled()) logger.info(" >");
        }
    }

    protected void showScheduleActions(String[] actions) {
        if (logger.isDebugEnabled()) logger.info("\tACTIONS: < ");
        if (actions != null && actions.length > 0) {
            for (int i = 0; i < actions.length - 1; i++)
                if (logger.isDebugEnabled()) logger.info(actions[i] + ",");
            if (logger.isDebugEnabled()) logger.info("" + actions[actions.length - 1] + " >");
        } else {
            if (logger.isDebugEnabled()) logger.info(" >");
        }
    }

    protected void showState(Event ev) {
        StateProvider state = (StateProvider) ev;
        StateInfo info = state.getState();
        if (info != null) showState(info);
    }

    protected void showState(StateInfo info) {
        if (logger.isDebugEnabled()) logger.info("STATE:");
        if (logger.isDebugEnabled()) logger.info("\tCHRONO:" + info.getChrono());
        if (logger.isDebugEnabled()) logger.info("\tDEPTH:" + info.getDepth());
        if (logger.isDebugEnabled()) logger.info("\tCURRENT NODE:" + info.getCurrentNode());
        VariableListInfo variables = info.getVariableList();
        if (variables != null) {
            if (logger.isDebugEnabled()) logger.info("\t");
            showVariables(variables);
        }
        ConstraintListInfo constraints = info.getConstraintList();
        if (constraints != null) {
            if (logger.isDebugEnabled()) logger.info("\t");
            showContraints(constraints);
        }
        UpdateListInfo updates = info.getUpdateList();
        if (updates != null) {
            if (logger.isDebugEnabled()) logger.info("\t");
            showUpdates(updates);
        }
        StateMiscInfo misc = info.getMisc();
        if (misc != null) if (logger.isDebugEnabled()) logger.info("\tMISC:" + misc.getMisc());
    }

    protected void showVariables(VariableListInfo variables) {
        VariableInfo info;
        Iterator it = (variables == null ? null : variables.iterator());
        if (logger.isDebugEnabled()) logger.info("STATE VARIABLES:< ");
        while (it != null && it.hasNext()) {
            info = (VariableInfo) it.next();
            if (logger.isDebugEnabled()) logger.info("\t");
            showVariable(info);
        }
        if (logger.isDebugEnabled()) logger.info(" >");
    }

    protected void showVariable(VariableInfo info) {
        if (logger.isDebugEnabled()) logger.info("\tSTATE VARIABLE:" + info.getIdentifier());
        if (logger.isDebugEnabled()) logger.info("\tNAME:" + info.getName());
        if (logger.isDebugEnabled()) logger.info("\tINTERNAL:" + info.getInternal());
        if (logger.isDebugEnabled()) logger.info("\tEXTERNAL:" + info.getExternal());
        ValueListInfo valueList = info.getValueList();
        if (valueList != null) {
            if (logger.isDebugEnabled()) logger.info("\t");
            showValueList(valueList);
        }
        VarDomainInfo varDomain = info.getVarDomain();
        if (varDomain != null) {
            if (logger.isDebugEnabled()) logger.info("\t");
            showVarDomain(varDomain);
        }
    }

    protected void showContraints(ConstraintListInfo constraints) {
        ConstraintInfo info;
        Iterator it = (constraints == null ? null : constraints.iterator());
        if (logger.isDebugEnabled()) logger.info("STATE CONSTRAINTS:< ");
        while (it != null && it.hasNext()) {
            info = (ConstraintInfo) it.next();
            if (logger.isDebugEnabled()) logger.info("\t");
            showConstraint(info);
        }
        if (logger.isDebugEnabled()) logger.info(" >");
    }

    protected void showConstraint(ConstraintInfo info) {
        if (logger.isDebugEnabled()) logger.info("\tSTATE CONSTRAINT:" + info.getIdentifier());
        if (logger.isDebugEnabled()) logger.info("\tNAME:" + info.getName());
        if (logger.isDebugEnabled()) logger.info("\tINTERNAL:" + info.getInternal());
        if (logger.isDebugEnabled()) logger.info("\tEXTERNAL:" + info.getExternal());
        UpdateListInfo updateList = info.getUpdateList();
        if (updateList != null) {
            if (logger.isDebugEnabled()) logger.info("\t");
            showUpdates(updateList);
        }

    }
*/
    	public void show() {
    		if (logger.isDebugEnabled()) logger.info("Loaded");
    	}
    

	/**
	 * 
	 */
	public SolverManagerEventListener(Solver solver) {
		super(solver);
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.parsing.event.EventListener#processEnd()
	 */
	public void processEnd() {
		super.processEnd();
		solver.setIntValue(Solver.CHRONO_MIN, chrono_min); 
		solver.setIntValue(Solver.CHRONO_MAX, chrono_max); 
		solver.setIntValue(Solver.DEPTH_MIN, depth_min); 
		solver.setIntValue(Solver.DEPTH_MAX, depth_max); 
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.parsing.event.EventListener#processStart()
	 */
	public void processStart() {
		super.processStart();
		chrono_min = Integer.MAX_VALUE;
		chrono_max = Integer.MIN_VALUE;
		depth_min = Integer.MAX_VALUE;
		depth_max = Integer.MIN_VALUE;
		
		valueListAdapter = new DefaultValueListAdapter(solver);
		domainAdapter = new DefaultDomainAdapter(solver);
		deltaAdapter = new DefaultDeltaAdapter(solver);
		variableAdapter = new DefaultVariableAdapter(solver);
		constraintAdapter = new DefaultConstraintAdapter(solver);
		searchNodeAdapter = new DefaultSearchNodeAdapter(solver);
		variableStatusAdapter = new DefaultVariableStatusAdapter(solver);
		stateAdapter = new DefaultStateAdapter(solver);
		reduceAdapter = new DefaultReduceAdapter(solver);
		updateAdapter = new DefaultUpdateAdapter(solver);
		constraintStatusAdapter = new DefaultConstraintStatusAdapter(solver);
		postAdapter = new DefaultPostAdapter(solver);
		awakeAdapter = new DefaultAwakeAdapter(solver);
		explanationAdapter =new DefaultExplanationAdapter(solver);
		causeAdapter = new DefaultCauseAdapter(solver);
		
		domainAdapter.setValueListAdapter(valueListAdapter);
		
		deltaAdapter.setValueListAdapter(valueListAdapter);
		
		variableAdapter.setDomainAdapter(domainAdapter);
		variableAdapter.setStateAdapter(stateAdapter);
		
		constraintAdapter.setStateAdapter(stateAdapter);
		
		postAdapter.setStateAdapter(stateAdapter);
		
		awakeAdapter.setStateAdapter(stateAdapter);
		awakeAdapter.setUpdateAdapter(updateAdapter);
		
		reduceAdapter.setDeltaAdapter(deltaAdapter);
		reduceAdapter.setDomainAdapter(domainAdapter);
		reduceAdapter.setStateAdapter(stateAdapter);
		reduceAdapter.setUpdateAdapter(updateAdapter);
		reduceAdapter.setExplanationAdapter(explanationAdapter);
		
		explanationAdapter.setValueListAdapter(valueListAdapter);
		explanationAdapter.setCauseAdapter(causeAdapter);
		
		stateAdapter.setVariableAdapter(variableStatusAdapter);
		stateAdapter.setConstraintAdapter(constraintStatusAdapter);
		stateAdapter.setUpdateAdapter(updateAdapter);
	}
	
}
