/*
 * INRIA-OADYMPPAC
 * 
 * Ownership: Copyright (c) 1990-2004 COSYTEC S.A.
 * Author	: 	raf
 *           	COSYTEC
 *		    	4 rue Jean Rostand
 * 				91893 Orsay Cedex, France
 *				Tel. +33 1 60 19 37 38
 *				Fax. +33 1 60 19 36 20
 *
 * Purpose	: COMPLETE HERE
 * 
 * Created on 17 mars 2004
 *
 * 
 */
package fr.inria.oadymppac.solver.parsing;

import infovis.Graph;

import org.apache.log4j.Logger;

import fr.inria.oadymppac.parsing.gentra4cp.event.NewConstraintEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.NewVariableEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.ReduceEvent;
import fr.inria.oadymppac.parsing.info.NewConstraintInfo;
import fr.inria.oadymppac.parsing.info.NewVariableInfo;
import fr.inria.oadymppac.parsing.info.ReduceInfo;
import fr.inria.oadymppac.solver.Constraint;
import fr.inria.oadymppac.solver.ConstraintManager;
import fr.inria.oadymppac.solver.ConstraintProxy;
import fr.inria.oadymppac.solver.Reduce;
import fr.inria.oadymppac.solver.ReduceManager;
import fr.inria.oadymppac.solver.Solver;
import fr.inria.oadymppac.solver.Variable;
import fr.inria.oadymppac.solver.VariableManager;
import fr.inria.oadymppac.solver.VariableProxy;
import fr.inria.oadymppac.solver.analyser.VariableConstraintGraphAnalyser;

/**
 * @author raf
 *  @
 */
public class VariableConstraintManagerEventListener extends AbstractManagerEventListener { 
	public static Logger logger = Logger.getLogger(VariableConstraintManagerEventListener.class);
	
	VariableConstraintGraphAnalyser analyser;
	
	private boolean performEndAnalyze=true;

    public void eventProcess(NewVariableEvent ev) {
    	super.eventProcess(ev);
    	try {
    		NewVariableInfo info = (NewVariableInfo) ev.getInfo();
    		int var=solver.get(Solver.VARIABLE,VariableManager.key(extractVariableIdentifier(info.getIdentifier())));   		
    		if (var != VariableManager.INVALID) {
    			Variable variable = (Variable) solver.get(Solver.VARIABLE, var);
    			if (variable != null && !(variable instanceof VariableProxy))analyser.add(variable);
    		} else {
   				if (logger.isDebugEnabled()) logger.warn("Impossible to get corresponding variable "+info.getIdentifier());
   			}
    	} catch (Exception exp) {
    		solver.add(Solver.EXCEPTION,exp);
    	}
    }

    public void eventProcess(NewConstraintEvent ev) {
    	super.eventProcess(ev);
    	try {    		
    		NewConstraintInfo info = (NewConstraintInfo) ev.getInfo();
    		int ctr=solver.get(Solver.CONSTRAINT, ConstraintManager.key(extractConstraintIdentifier(info.getIdentifier())));
    		if (ctr != ConstraintManager.INVALID) {
    			Constraint constraint = (Constraint)solver.get(Solver.CONSTRAINT, ctr);
    			if (constraint != null && !(constraint instanceof ConstraintProxy))analyser.add(constraint);
     		} else {
   				if (logger.isDebugEnabled()) logger.warn("Impossible to get corresponding constraint "+info.getIdentifier());
   			}
    	} catch (Exception exp) {
    		solver.add(Solver.EXCEPTION,exp);
    	}
        
    }

    public void eventProcess(ReduceEvent ev) {
    	super.eventProcess(ev);
    	try {
    		
    		ReduceInfo info = (ReduceInfo) ev.getInfo();
    		int red=solver.get(Solver.REDUCE,ReduceManager.key(extractEventIdentifier(REDUCE,info.getChrono())));
    		int weight=0; // weight to add to the link 
    		if (red != ReduceManager.INVALID) {
    			Reduce reduce = (Reduce)solver.get(Solver.REDUCE,red);
    			if (reduce != null) analyser.add(reduce);
   			} else {
   				if (logger.isDebugEnabled())  {
   					logger.warn("Impossible to get corresponding reduce at chrono "+info.getChrono());
   					logger.warn("using default Reduce "+extractEventIdentifier(REDUCE,info.getChrono()) );
   				}
   			}
    	} catch (Exception exp) {
    		solver.add(Solver.EXCEPTION, exp);
    	}
    }
    
    
    
	/**
	 * @return Returns the solver.
	 */
	public Solver getSolver() {
		return solver;
	}
	/**
	 * @param solver The solver to set.
	 */
	public void setSolver(Solver solver) {
		this.solver = solver;
	}
	/**
	 * 
	 */
	public VariableConstraintManagerEventListener(Solver solver) {
		super(solver);
		setAnalyser(new VariableConstraintGraphAnalyser(solver));
		
	}
	/**
	 * @return Returns the graph.
	 */
	public Graph getGraph() {
		return analyser.getGraph();
	}

	/**
	 * @return Returns the analyser.
	 */
	public synchronized VariableConstraintGraphAnalyser getAnalyser() {
		return analyser;
	}
	/**
	 * @param analyser The analyser to set.
	 */
	public synchronized void setAnalyser(
			VariableConstraintGraphAnalyser analyser) {
		this.analyser = analyser;
	}
	
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.parsing.event.EventListener#processEnd()
	 */
	public void processStart() {
		performEndAnalyze=true;
	};
	
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.parsing.event.EventListener#processEnd()
	 */
	public void processEnd() {
		if (analyser != null && performEndAnalyze) {
			analyser.endAnalyse();
			performEndAnalyze=false;
		}
	}
}
