/* 
 * ### Copyright (C) 2001-2003 Michael Fuchs ###
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *      
 * Author: Michael Fuchs
 * E-Mail: mfuchs@unico-consulting.com
 *
 * RCS Information:
 * ---------------
 * Id.........: $Id: PathOption.java, v 1.1.1.1 2003/10/21 19:19:15 cvs Exp $
 * Author.....: $Author: cvs $
 * Date.......: $Date: 2003/10/21 19:19:15 $
 * Revision...: $Revision: 1.1.1.1 $
 * State......: $State: Exp $
 */
package com.mf.option;

import java.io.File;
import java.util.StringTokenizer;

/**
 * Describe class <code>PathOption</code> here.
 *
 * @author <a href="mailto:mfuchs@unico-consulting.com">Michael Fuchs</a>
 * @version 1.0
 */
public class PathOption extends Option {

    /**
     * Describe variable <code>invalidPathElement</code> here.
     *
     */
    private String invalidPathElement = "";

    /**
     * Creates a new <code>PathOption</code> instance.
     *
     */
    public PathOption() {
        super();
        hasArgument(true);
    }

    /**
     * Creates a new <code>PathOption</code> instance.
     *
     * @param longName a <code>String</code> value
     */
    public PathOption(String longName) {
        super(longName);
        hasArgument(true);
    }

    /**
     * Creates a new <code>PathOption</code> instance.
     *
     * @param longName a <code>String</code> value
     * @param shortName a <code>String</code> value
     */
    public PathOption(String longName, String shortName) {
        super(longName, shortName);
        hasArgument(true);
    }

    /**
     * Describe <code>getInvalidPathElement</code> method here.
     *
     * @return a <code>String</code> value
     */
    public String getInvalidPathElement() {
        return invalidPathElement;
    }
    
    /**
     * Describe <code>checkArgument</code> method here.
     *
     * @param arg a <code>String</code> value
     * @return a <code>boolean</code> value
     */
    protected boolean checkArgument(String arg) {
        StringTokenizer stz = new StringTokenizer
            (arg, System.getProperty("path.separator"));

        String pathElement;
        File path;

        while (stz.hasMoreTokens()) {
            
            pathElement = stz.nextToken();
            path = new File(pathElement);
            
            if (path.exists() == false) {
                invalidPathElement = pathElement;
                return false;
            } // end of if (path.exists() == false)
            
        } // end of while (stz.hasMoreTokens()))
    
        return true;
    }
}
/*
 * $Log: PathOption.java, v $
 * Revision 1.1.1.1  2003/10/21 19:19:15  cvs
 * Getopts library.
 *
 * Revision 1.1  2003/08/13 18:39:47  mfuchs
 * Option
 *
 * Revision 1.1  2003/08/12 19:28:10  mfuchs
 * log4j properties files for differnet logging levels.
 *
 */
